// -------------------------------------------------------------------------------- 
//!  @file  main.c                                                                  
//!  メインのソースファイル                                                         
//                                                                                  
//   Copyright (C) 2022 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"
#include "menu.h"


// -------------------------------------------------------------------------------- 
//!  ラピッドファイア更新                                                           
//!                                                                                 
//!  @param [in,out] p_rapidFire  ラピッドファイア                                  
//!  @param [in]     index        インデックス                                      
//!  @return                      なし                                              
// -------------------------------------------------------------------------------- 
static void RAMFUNC updateRapidFire(RAPIDFIRE *p_rapidFire, int index)
{
	uint16_t	button ;


	if( !p_rapidFire->button ) {
		return ;
	}

	button = (1 << p_rapidFire->button) ;
	if( g.pad.once & button ) {
		p_rapidFire->count = 0 ;
	} else
	if( g.pad.norm & button ) {
		if( ++p_rapidFire->count > 2 ) {
			p_rapidFire->count = 0 ;
			g.pad.norm        |= (1<<index) ;
		}
	}
}


// -------------------------------------------------------------------------------- 
//!  メイン処理                                                                     
//!                                                                                 
//!  @param [in] argc  引数の数                                                     
//!  @param [in] argv  引数                                                         
//!  @return           0                                                            
// -------------------------------------------------------------------------------- 
int RAMFUNC main(int argc, char *argv[])
{
	set_sys_clock_khz(200000, true) ;

	setup_default_uart() ;

	common_init() ;

	sm3gg_setRomInfo( &g_romInfo[0] ) ;

	while( 1 ) {
		vga_waitVSync() ;

		sound_update() ;

		vga_setBgBlank() ;

		if( g.borderColor ) {
			vga_setColor( 0x0400 ) ;
		} else {
			vga_setColor( g.p_vram->borderColor ) ;
		}

		g.pad.old    = g.pad.norm ;
		g.pad.norm   = g_padInfo.status ;
		g.pad.once   = (g.pad.norm ^ g.pad.old) & g.pad.norm ;
		g.pad.analog = g_padInfo.analog[0] ;

		// メニュー処理
		if( g.menu ) {
			menuProc() ;
			continue ;
		}

		// メニュー表示
		if( g.pad.once & PAD_SELECT ) {
			g.menu = 1 ;
			g.proc = 0 ;
			sound_mute( 1 ) ;
			continue ;
		}

		// ラピッドファイア
		updateRapidFire(&g.rapidFire[0], 0) ;
		updateRapidFire(&g.rapidFire[1], 1) ;

		g.p_vram = &g.vram[g.page] ;
		g.page  ^= 1 ;

		sm3gg_supplyClock() ;
	}

	return 0 ;
}
