// -------------------------------------------------------------------------------- 
//!  @file  vdp.h                                                                   
//!  VDP のヘッダーファイル                                                         
//                                                                                  
//   Copyright (C) 2022 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#ifndef __VDP_H__INCLUDED__
#define __VDP_H__INCLUDED__

	//! スプライト情報
	typedef struct {
		uint32_t					pattern[2] ;					//!< 描画パターン
		uint8_t						xpos ;							//!< ｘ座標
	} SPRITEINFO ;

	//! VDP
	typedef struct {
		uint8_t						status ;						//!< 状態
		uint8_t						write ;							//!< 書き込みフラグ
		uint8_t						writeData ;						//!< 書き込みデータ
		uint8_t						readData ;						//!< 読み込みデータ
		uint8_t						cmd ;							//!< コマンド
		uint8_t						vcount ;						//!< V カウント
		uint8_t						lineCount ;						//!< 行カウンタ
		uint8_t						blank ;							//!< ブランクフラグ
		uint8_t						nt ;							//!< ネームテーブル
		uint8_t						sx ;							//!< 水平スクロール
		uint8_t						sy ;							//!< 垂直スクロール
		uint8_t						colorGG ;						//!< GG 色下位
		uint32_t					borderColor ;					//!< ボーダー色
		uint16_t					address ;						//!< VRAM アドレス
		uint16_t					scanLine ;						//!< 走査線
		uint8_t						spriteInfoCount ;				//!< スプライト情報数
		SPRITEINFO					spriteInfo[8] ;					//!< スプライト情報
		uint32_t					beginDrawClock ;				//!< 描画開始時クロック
		uint16_t					palette[32] ;					//!< パレット
		uint16_t					const *p_lineInfo ;				//!< カレント行情報
		uint8_t						*p_image ;						//!< 画像アドレス
		uint8_t						regs[16] ;						//!< レジスタ
		uint8_t						vram[0x4000] ;					//!< VRAM
		PIN							*p_pin ;						//!< ピン
		ADDRESSBUS					addressBus ;					//!< アドレスバス
		CLOCKEVENT					clockEvent ;					//!< クロックイベント
		CLOCKEVENT					clockEventOverlap ;				//!< スプライト衝突フラグ用クロックイベント
	} VDP ;


extern void vdp_init(VDP *p, PIN *p_pin) ;
extern void vdp_reset( VDP *p ) ;


#endif//__VDP_H__INCLUDED__
