// -------------------------------------------------------------------------------- 
//!  @file  z80.c                                                                   
//!  Z80 のソースファイル                                                           
//                                                                                  
//   Copyright (C) 2022 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include "common.h"

//! 1CPU クロックあたりのクロックイベントクロック数
#define _CLK( n )					((n)*Z80_CLK)

//! コンディションコード
#define CC_T						1
#define CC_NZ						!(p->regs.b.f & FLAG_Z)
#define CC_Z						 (p->regs.b.f & FLAG_Z)
#define CC_NC						!(p->regs.b.f & FLAG_C)
#define CC_C						 (p->regs.b.f & FLAG_C)
#define CC_PO						!(p->regs.b.f & FLAG_P)
#define CC_PE						 (p->regs.b.f & FLAG_P)
#define CC_P						!(p->regs.b.f & FLAG_S)
#define CC_M						 (p->regs.b.f & FLAG_S)

//! 未定義命令
#define OP_UNDEF( num )																												\
		case num+0: { undefInstruction( p ) ;                                                                return _CLK(4) ; }		\
		case num+1: { undefInstruction( p ) ;                                                                return _CLK(4) ; }		\
		case num+2: { undefInstruction( p ) ;                                                                return _CLK(4) ; }		\
		case num+3: { undefInstruction( p ) ;                                                                return _CLK(4) ; }		\
		case num+4: { undefInstruction( p ) ;                                                                return _CLK(4) ; }		\
		case num+5: { undefInstruction( p ) ;                                                                return _CLK(4) ; }		\
		case num+6: { undefInstruction( p ) ;                                                                return _CLK(4) ; }		\
		case num+7: { undefInstruction( p ) ;                                                                return _CLK(4) ; }

//! CB - ROTATE/SHIFT
#define CB_RS(num, inst)																											\
		case num+0: { p->regs.b.b = inst(p, p->regs.b.b) ;                                                   return _CLK(8) ;  }	\
		case num+1: { p->regs.b.c = inst(p, p->regs.b.c) ;                                                   return _CLK(8) ;  }	\
		case num+2: { p->regs.b.d = inst(p, p->regs.b.d) ;                                                   return _CLK(8) ;  }	\
		case num+3: { p->regs.b.e = inst(p, p->regs.b.e) ;                                                   return _CLK(8) ;  }	\
		case num+4: { p->regs.b.h = inst(p, p->regs.b.h) ;                                                   return _CLK(8) ;  }	\
		case num+5: { p->regs.b.l = inst(p, p->regs.b.l) ;                                                   return _CLK(8) ;  }	\
		case num+6: { writeByte(p, p->regs.w.hl, inst(p, readByte(p, p->regs.w.hl)))  ;                      return _CLK(15) ; }	\
		case num+7: { p->regs.b.a = inst(p, p->regs.b.a) ;                                                   return _CLK(8) ;  }

//! CB - BIT
#define CB_BIT(num, bit)																											\
		case num+0: { _bit_r(p, bit, p->regs.b.b) ;                                                          return _CLK(8) ;  }	\
		case num+1: { _bit_r(p, bit, p->regs.b.c) ;                                                          return _CLK(8) ;  }	\
		case num+2: { _bit_r(p, bit, p->regs.b.d) ;                                                          return _CLK(8) ;  }	\
		case num+3: { _bit_r(p, bit, p->regs.b.e) ;                                                          return _CLK(8) ;  }	\
		case num+4: { _bit_r(p, bit, p->regs.b.h) ;                                                          return _CLK(8) ;  }	\
		case num+5: { _bit_r(p, bit, p->regs.b.l) ;                                                          return _CLK(8) ;  }	\
		case num+6: { _bit_hl(p, bit, readByte(p, p->regs.w.hl)) ;                                           return _CLK(12) ; }	\
		case num+7: { _bit_r(p, bit, p->regs.b.a) ;                                                          return _CLK(8) ;  }

//! CB - RES/SET
#define CB_BRS(num, bit, inst)																										\
		case num+0: { p->regs.b.b = inst(bit, p->regs.b.b) ;                                                 return _CLK(8) ;  }	\
		case num+1: { p->regs.b.c = inst(bit, p->regs.b.c) ;                                                 return _CLK(8) ;  }	\
		case num+2: { p->regs.b.d = inst(bit, p->regs.b.d) ;                                                 return _CLK(8) ;  }	\
		case num+3: { p->regs.b.e = inst(bit, p->regs.b.e) ;                                                 return _CLK(8) ;  }	\
		case num+4: { p->regs.b.h = inst(bit, p->regs.b.h) ;                                                 return _CLK(8) ;  }	\
		case num+5: { p->regs.b.l = inst(bit, p->regs.b.l) ;                                                 return _CLK(8) ;  }	\
		case num+6: { writeByte(p, p->regs.w.hl, inst(bit, readByte(p, p->regs.w.hl)))  ;                    return _CLK(12) ; }	\
		case num+7: { p->regs.b.a = inst(bit, p->regs.b.a) ;                                                 return _CLK(8) ;  }

//! DD/FD CB - ROTATE/SHIFT
#define XDCB_RS(num, address, inst)																									\
		case num+0: { p->regs.b.b = inst(p, readByte(p, address)) ;  writeByte(p, address, p->regs.b.b) ;    return _CLK(23) ; }	\
		case num+1: { p->regs.b.c = inst(p, readByte(p, address)) ;  writeByte(p, address, p->regs.b.c) ;    return _CLK(23) ; }	\
		case num+2: { p->regs.b.d = inst(p, readByte(p, address)) ;  writeByte(p, address, p->regs.b.d) ;    return _CLK(23) ; }	\
		case num+3: { p->regs.b.e = inst(p, readByte(p, address)) ;  writeByte(p, address, p->regs.b.e) ;    return _CLK(23) ; }	\
		case num+4: { p->regs.b.h = inst(p, readByte(p, address)) ;  writeByte(p, address, p->regs.b.h) ;    return _CLK(23) ; }	\
		case num+5: { p->regs.b.l = inst(p, readByte(p, address)) ;  writeByte(p, address, p->regs.b.l) ;    return _CLK(23) ; }	\
		case num+6: { writeByte(p, address, inst(p, readByte(p, address))) ;                                 return _CLK(23) ; }	\
		case num+7: { p->regs.b.a = inst(p, readByte(p, address)) ;  writeByte(p, address, p->regs.b.a) ;    return _CLK(23) ; }

//! DD/FD CB - BIT
#define XDCB_BIT(num, address, bit)																									\
		case num+0: { _bit_m(p, bit, readByte(p, address)) ;                                                 return _CLK(20) ; }	\
		case num+1: { _bit_m(p, bit, readByte(p, address)) ;                                                 return _CLK(20) ; }	\
		case num+2: { _bit_m(p, bit, readByte(p, address)) ;                                                 return _CLK(20) ; }	\
		case num+3: { _bit_m(p, bit, readByte(p, address)) ;                                                 return _CLK(20) ; }	\
		case num+4: { _bit_m(p, bit, readByte(p, address)) ;                                                 return _CLK(20) ; }	\
		case num+5: { _bit_m(p, bit, readByte(p, address)) ;                                                 return _CLK(20) ; }	\
		case num+6: { _bit_m(p, bit, readByte(p, address)) ;                                                 return _CLK(20) ; }	\
		case num+7: { _bit_m(p, bit, readByte(p, address)) ;                                                 return _CLK(20) ; }

//! DD/FD CB - RES/SET
#define XDCB_BRS(num, address, bit, inst)																							\
		case num+0: { p->regs.b.b = inst(bit, readByte(p, address)) ;  writeByte(p, address, p->regs.b.b) ;  return _CLK(23) ; }	\
		case num+1: { p->regs.b.c = inst(bit, readByte(p, address)) ;  writeByte(p, address, p->regs.b.c) ;  return _CLK(23) ; }	\
		case num+2: { p->regs.b.d = inst(bit, readByte(p, address)) ;  writeByte(p, address, p->regs.b.d) ;  return _CLK(23) ; }	\
		case num+3: { p->regs.b.e = inst(bit, readByte(p, address)) ;  writeByte(p, address, p->regs.b.e) ;  return _CLK(23) ; }	\
		case num+4: { p->regs.b.h = inst(bit, readByte(p, address)) ;  writeByte(p, address, p->regs.b.h) ;  return _CLK(23) ; }	\
		case num+5: { p->regs.b.l = inst(bit, readByte(p, address)) ;  writeByte(p, address, p->regs.b.l) ;  return _CLK(23) ; }	\
		case num+6: { writeByte(p, address, inst(bit, readByte(p, address))) ;                               return _CLK(23) ; }	\
		case num+7: { p->regs.b.a = inst(bit, readByte(p, address)) ;  writeByte(p, address, p->regs.b.a) ;  return _CLK(23) ; }

	//! 状態
	enum {
		STATUS_HALT_				=	0,							//!< HALT 中
		STATUS_NMI_					=	1,							//!< NMI 発生
		STATUS_INT_					=	2,							//!< INT 発生
		STATUS_INTMASK_				=	3,							//!< 割り込みマスク
		STATUS_REQ_					=	4,							//!< 割り込みリクエスト
		STATUS_HALT					=	1 << STATUS_HALT_,
		STATUS_NMI					=	1 << STATUS_NMI_,
		STATUS_INT					=	1 << STATUS_INT_,
		STATUS_INTMASK				=	1 << STATUS_INTMASK_,
		STATUS_NMIREQ				=	1 << (STATUS_NMI_+STATUS_REQ_),
		STATUS_INTREQ				=	1 << (STATUS_INT_+STATUS_REQ_)
	} ;

	//! フラグ
	enum {
		FLAG_S_						=	7,							//!< サインフラグ
		FLAG_Z_						=	6,							//!< ゼロフラグ
		FLAG_Y_						=	5,							//!< 未使用
		FLAG_H_						=	4,							//!< ハーフキャリーフラグ
		FLAG_X_						=	3,							//!< 未使用
		FLAG_V_						=	2,							//!< オーバーフローフラグ
		FLAG_P_						=	FLAG_V_,					//!< パリティフラグ
		FLAG_N_						=	1,							//!< 減算フラグ
		FLAG_C_						=	0,							//!< キャリーフラグ
		FLAG_S						=	1 << FLAG_S_,
		FLAG_Z						=	1 << FLAG_Z_,
		FLAG_Y						=	1 << FLAG_Y_,
		FLAG_H						=	1 << FLAG_H_,
		FLAG_X						=	1 << FLAG_X_,
		FLAG_V						=	1 << FLAG_V_,
		FLAG_P						=	1 << FLAG_P_,
		FLAG_N						=	1 << FLAG_N_,
		FLAG_C						=	1 << FLAG_C_
	} ;

	//!< 割り込みイネーブルフリップフロップ
	enum {
		IFF1						=	1 << (FLAG_P_+1),
		IFF2						=	1 << (FLAG_P_+0)
	} ;


// -------------------------------------------------------------------------------- 
//!  指定アドレスからバイトデータを読み込む                                         
//!                                                                                 
//!  @param [in] p        this ポインタ                                             
//!  @param [in] address  アドレス                                                  
//!  @return              データ                                                    
// -------------------------------------------------------------------------------- 
static inline int8_t RAMFUNC readByte(Z80 *p, uint16_t address)
{
	return readAddressBus(p->p_mem, address) ;
}


// -------------------------------------------------------------------------------- 
//!  指定アドレスからワードデータを読み込む                                         
//!                                                                                 
//!  @param [in] p        this ポインタ                                             
//!  @param [in] address  アドレス                                                  
//!  @return              データ                                                    
// -------------------------------------------------------------------------------- 
static inline uint16_t RAMFUNC readWord(Z80 *p, uint16_t address)
{
	uint16_t	data ;


	data  = ((uint16_t )readAddressBus(p->p_mem, address+0) << 0) ;
	data |= ((uint16_t )readAddressBus(p->p_mem, address+1) << 8) ;

	return data ;
}


// -------------------------------------------------------------------------------- 
//!  指定アドレスへバイトデータを書き込む                                           
//!                                                                                 
//!  @param [in] p        this ポインタ                                             
//!  @param [in] address  アドレス                                                  
//!  @param [in] data     データ                                                    
//!  @return              なし                                                      
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC writeByte(Z80 *p, uint16_t address, uint8_t data)
{
	writeAddressBus(p->p_mem, address, data) ;
}


// -------------------------------------------------------------------------------- 
//!  指定アドレスへワードデータを書き込む                                           
//!                                                                                 
//!  @param [in] p        this ポインタ                                             
//!  @param [in] address  アドレス                                                  
//!  @param [in] data     データ                                                    
//!  @return              なし                                                      
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC writeWord(Z80 *p, uint16_t address, uint16_t data)
{
	writeAddressBus(p->p_mem, address+0, (uint8_t )(data>>0)) ;
	writeAddressBus(p->p_mem, address+1, (uint8_t )(data>>8)) ;
}


// -------------------------------------------------------------------------------- 
//!  PC アドレスからバイトデータを読み込み PC を進める                              
//!                                                                                 
//!  @param [in,out] p  this ポインタ                                               
//!  @return            データ                                                      
// -------------------------------------------------------------------------------- 
static inline uint8_t RAMFUNC readBytePC( Z80 *p )
{
	return readAddressBus(p->p_mem, p->pc++) ;
}


// -------------------------------------------------------------------------------- 
//!  PC アドレスからワードデータを読み込み PC を進める                              
//!                                                                                 
//!  @param [in,out] p  this ポインタ                                               
//!  @return            データ                                                      
// -------------------------------------------------------------------------------- 
static inline uint16_t RAMFUNC readWordPC( Z80 *p )
{
	uint16_t	data ;


	data   = readWord(p, p->pc) ;
	p->pc += 2 ;

	return data ;
}


// -------------------------------------------------------------------------------- 
//!  インデックスレジスタにオフセットを足したデータを取得                           
//!                                                                                 
//!  @param [in,out] p  this ポインタ                                               
//!  @return            データ                                                      
// -------------------------------------------------------------------------------- 
static inline uint16_t RAMFUNC getId(Z80 *p, uint16_t src)
{
	p->calc = src + (int8_t )readBytePC( p ) ;

	return p->calc ;
}


// -------------------------------------------------------------------------------- 
//   未定義命令                                                                     
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC undefInstruction( Z80 *p )
{
	p->pc-- ;
}


// -------------------------------------------------------------------------------- 
//   LD      A, I/R                                                                 
// -------------------------------------------------------------------------------- 
static void RAMFUNC _lda(Z80 *p, uint8_t src)
{
	p->regs.b.a = src ;
	p->regs.b.f = (p->flags[src]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X)) | (p->iff&IFF2) ;
}


// -------------------------------------------------------------------------------- 
//   CCF                                                                            
// -------------------------------------------------------------------------------- 
static inline uint32_t RAMFUNC _ccf( Z80 *p )
{
	p->regs.b.f = ((p->regs.b.f^FLAG_C)&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_V|FLAG_C)) | (p->regs.b.a&(FLAG_Y|FLAG_X)) | ((p->regs.b.f&FLAG_C)<<(FLAG_H_-FLAG_C_)) ;

	return _CLK(4) ;
}


// -------------------------------------------------------------------------------- 
//   SCF                                                                            
// -------------------------------------------------------------------------------- 
static inline uint32_t RAMFUNC _scf( Z80 *p )
{
	p->regs.b.f = (p->regs.b.f&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_V)) | (p->regs.b.a&(FLAG_Y|FLAG_X)) | FLAG_C ;

	return _CLK(4) ;
}


// -------------------------------------------------------------------------------- 
//   EX      dst, src                                                               
// -------------------------------------------------------------------------------- 
static inline uint16_t RAMFUNC _ex(uint16_t dst, uint16_t *p_src)
{
	uint16_t	src ;


	src    = *p_src ;
	*p_src = dst ;

	return src ;
}


// -------------------------------------------------------------------------------- 
//   EXX                                                                            
// -------------------------------------------------------------------------------- 
static inline uint32_t RAMFUNC _exx( Z80 *p )
{
	p->regs.w.bc = _ex(p->regs.w.bc, &p->regs.w.bc_) ;
	p->regs.w.de = _ex(p->regs.w.de, &p->regs.w.de_) ;
	p->regs.w.hl = _ex(p->regs.w.hl, &p->regs.w.hl_) ;

	return _CLK(4) ;
}


// -------------------------------------------------------------------------------- 
//   PUSH    src                                                                    
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC _push(Z80 *p, uint16_t src)
{
	p->regs.w.sp -= 2 ;
	writeWord(p, p->regs.w.sp, src) ;
}


// -------------------------------------------------------------------------------- 
//   POP     dst                                                                    
// -------------------------------------------------------------------------------- 
static inline uint16_t RAMFUNC _pop( Z80 *p )
{
	uint16_t	dst ;


	dst           = readWord(p, p->regs.w.sp) ;
	p->regs.w.sp += 2 ;

	return dst ;
}


// -------------------------------------------------------------------------------- 
//   LDI / LDD                                                                      
// -------------------------------------------------------------------------------- 
static void RAMFUNC _ldn(Z80 *p, int16_t add)
{
	uint8_t		res ;


	res = readByte(p, p->regs.w.hl) ;

	writeByte(p, p->regs.w.de, res) ;
	p->regs.w.de = p->regs.w.de + add ;
	p->regs.w.hl = p->regs.w.hl + add ;
	p->regs.w.bc-- ;

	res += p->regs.b.a ;

	p->regs.b.f = (p->regs.b.f&(FLAG_S|FLAG_Z|FLAG_C)) | ((uint8_t )(p->regs.w.bc!=0)<<FLAG_V_) | ((res<<(FLAG_Y_-1))&FLAG_Y) | (res&FLAG_X) ;
}


// -------------------------------------------------------------------------------- 
//   LDIR / LDDR                                                                    
// -------------------------------------------------------------------------------- 
static uint32_t RAMFUNC _ldnr(Z80 *p, int16_t add)
{
	_ldn(p, add) ;

	if( !p->regs.w.bc ) {
		return _CLK(16) ;
	}

	p->pc -= 2 ;

	return _CLK(21) ;
}


// -------------------------------------------------------------------------------- 
//   CPI / CPD                                                                      
// -------------------------------------------------------------------------------- 
static void RAMFUNC _cpn(Z80 *p, int16_t add)
{
	uint8_t		res, src ;


	src          = readByte(p, p->regs.w.hl) ;
	res          = p->regs.b.a - src ;
	p->regs.w.hl = p->regs.w.hl + add ;
	p->regs.w.bc-- ;

	p->regs.b.f = (uint8_t )((p->flags[res]&(FLAG_S|FLAG_Z|FLAG_N)) | (((p->regs.b.a^src)^res)&FLAG_H) | ((p->regs.w.bc!=0)<<FLAG_V_) | (p->regs.b.f&FLAG_C)) ;

	if( p->regs.b.f & FLAG_H ) {
		res-- ;
	}

	p->regs.b.f |= ((res<<(FLAG_Y_-1))&FLAG_Y) | (res&FLAG_X) ;
}


// -------------------------------------------------------------------------------- 
//   CPIR / CPDR                                                                    
// -------------------------------------------------------------------------------- 
static uint32_t RAMFUNC _cpnr(Z80 *p, int16_t add)
{
	_cpn(p,  add) ;

	if( (p->regs.b.f&FLAG_Z) || !p->regs.w.bc ) {
		return _CLK(16) ;
	}

	p->pc -= 2 ;

	return _CLK(21) ;
}


// -------------------------------------------------------------------------------- 
//   ADD     dst, src                                                               
// -------------------------------------------------------------------------------- 
static uint8_t RAMFUNC _addb(Z80 *p, uint8_t dst, uint8_t src)
{
	uint32_t	res ;


	res         = dst + src ;
	p->regs.b.f = (uint8_t )((p->flags[(uint8_t )res]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X)) | (((dst^src)^res)&FLAG_H) | ((((res^dst)&(res^src))>>(7-FLAG_V_))&FLAG_V) | (res>>8)) ;

	return (uint8_t )res ;
}

static uint16_t RAMFUNC _addw(Z80 *p, uint16_t dst, uint16_t src)
{
	uint32_t	res ;


	res         = dst + src ;
	p->regs.b.f = (uint8_t )((p->regs.b.f&(FLAG_S|FLAG_Z|FLAG_V)) | ((((dst^src)^res)>>8)&FLAG_H) | ((res>>8)&(FLAG_Y|FLAG_X)) | (res>>16)) ;

	return (uint16_t )res ;
}


// -------------------------------------------------------------------------------- 
//   ADC     dst, src                                                               
// -------------------------------------------------------------------------------- 
static uint8_t RAMFUNC _adcb(Z80 *p, uint8_t dst, uint8_t src)
{
	uint32_t	res ;


	res         = dst + src + (p->regs.b.f&FLAG_C) ;
	p->regs.b.f = (uint8_t )((p->flags[(uint8_t )res]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X)) | (((dst^src)^res)&FLAG_H) | ((((res^dst)&(res^src))>>(7-FLAG_V_))&FLAG_V) | (res>>8)) ;

	return (uint8_t )res ;
}

static uint16_t RAMFUNC _adcw(Z80 *p, uint16_t dst, uint16_t src)
{
	uint32_t	res ;


	res         = dst + src + (p->regs.b.f&FLAG_C) ;
	p->regs.b.f = (uint8_t )(((res>>8)&(FLAG_S|FLAG_Y|FLAG_X)) | (((uint16_t )res==0)<<FLAG_Z_) | ((((dst^src)^res)>>8)&FLAG_H) | ((((res^dst)&(res^src))>>(15-FLAG_V_))&FLAG_V) | (res>>16)) ;

	return (uint16_t )res ;
}


// -------------------------------------------------------------------------------- 
//   SUB     dst, src                                                               
// -------------------------------------------------------------------------------- 
static uint8_t RAMFUNC _sub(Z80 *p, uint8_t dst, uint8_t src)
{
	uint32_t	res ;


	res         = dst - src ;
	p->regs.b.f = (uint8_t )((p->flags[(uint8_t )res]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_N)) | (((dst^src)^res)&FLAG_H) | ((((res^dst)&(dst^src))>>(7-FLAG_V_))&FLAG_V) | (res>>31)) ;

	return (uint8_t )res ;
}


// -------------------------------------------------------------------------------- 
//   SBC     dst, src                                                               
// -------------------------------------------------------------------------------- 
static uint8_t RAMFUNC _sbcb(Z80 *p, uint8_t dst, uint8_t src)
{
	uint32_t	res ;


	res         = dst - src - (p->regs.b.f&FLAG_C) ;
	p->regs.b.f = (uint8_t )((p->flags[(uint8_t )res]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_N)) | (((dst^src)^res)&FLAG_H) | ((((res^dst)&(dst^src))>>(7-FLAG_V_))&FLAG_V) | (res>>31)) ;

	return (uint8_t )res ;
}

static uint16_t RAMFUNC _sbcw(Z80 *p, uint16_t dst, uint16_t src)
{
	uint32_t	res ;


	res         = dst - src - (p->regs.b.f&FLAG_C) ;
	p->regs.b.f = (uint8_t )(((res>>(15-FLAG_S_))&FLAG_S) | (((res<<16)==0)<<FLAG_Z_) | ((((dst^src)^res)>>8)&FLAG_H) | ((((res^dst)&(dst^src))>>(15-FLAG_V_))&FLAG_V) | FLAG_N | ((res>>8)&(FLAG_Y|FLAG_X)) | (res>>31)) ;

	return (uint16_t )res ;
}


// -------------------------------------------------------------------------------- 
//   CP      dst, src                                                               
// -------------------------------------------------------------------------------- 
static void RAMFUNC _cp(Z80 *p, uint8_t dst, uint8_t src)
{
	uint32_t	res ;


	res         = dst - src ;
	p->regs.b.f = (uint8_t )((p->flags[(uint8_t )res]&(FLAG_S|FLAG_Z|FLAG_N)) | (((dst^src)^res)&FLAG_H) | ((((res^dst)&(dst^src))>>(7-FLAG_V_))&FLAG_V) | (res>>31)) | (src&(FLAG_Y|FLAG_X)) ;
}


// -------------------------------------------------------------------------------- 
//   AND     dst, src                                                               
// -------------------------------------------------------------------------------- 
static uint8_t RAMFUNC _and(Z80 *p, uint8_t dst, uint8_t src)
{
	dst        &= src ;
	p->regs.b.f = p->flags[dst]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_H|FLAG_X|FLAG_P) ;

	return dst ;
}


// -------------------------------------------------------------------------------- 
//   OR      dst, src                                                               
// -------------------------------------------------------------------------------- 
static uint8_t RAMFUNC _or(Z80 *p, uint8_t dst, uint8_t src)
{
	dst        |= src ;
	p->regs.b.f = p->flags[dst]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_P) ;

	return dst ;
}


// -------------------------------------------------------------------------------- 
//   XOR     dst, src                                                               
// -------------------------------------------------------------------------------- 
static uint8_t RAMFUNC _xor(Z80 *p, uint8_t dst, uint8_t src)
{
	dst        ^= src ;
	p->regs.b.f = p->flags[dst]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_P) ;

	return dst ;
}


// -------------------------------------------------------------------------------- 
//   NEG                                                                            
// -------------------------------------------------------------------------------- 
static uint32_t RAMFUNC _neg( Z80 *p )
{
	p->regs.b.a = _sub(p, 0, p->regs.b.a) ;

	return _CLK(8) ;
}


// -------------------------------------------------------------------------------- 
//   BIT     num, src                                                               
// -------------------------------------------------------------------------------- 
static void RAMFUNC _bit_r(Z80 *p, uint8_t num, uint8_t src)
{
	uint8_t		z ;


	z           = ~(src >> num) & 1 ;
	p->regs.b.f = (uint8_t )((z<<FLAG_Z_) | FLAG_H | (z<<FLAG_P_) | (p->regs.b.f&FLAG_C) | ((src&(1<<num))&(FLAG_S|FLAG_Y|FLAG_X))) ;
}

static void RAMFUNC _bit_m(Z80 *p, uint8_t num, uint8_t src)
{
	uint8_t		z ;


	z           = ~(src >> num) & 1 ;
	p->regs.b.f = (uint8_t )((z<<FLAG_Z_) | FLAG_H | (z<<FLAG_P_) | (p->regs.b.f&FLAG_C) | ((src&(1<<num))&FLAG_S)) ;
}

static void RAMFUNC _bit_hl(Z80 *p, uint8_t num, uint8_t src)
{
	uint8_t		z ;


	z           = ~(src >> num) & 1 ;
	p->regs.b.f = (uint8_t )((z<<FLAG_Z_) | FLAG_H | (z<<FLAG_P_) | (p->regs.b.f&FLAG_C) | ((src&(1<<num))&FLAG_S) | ((p->calc>>8)&(FLAG_Y|FLAG_X))) ;
}


// -------------------------------------------------------------------------------- 
//   SET     num, dst                                                               
// -------------------------------------------------------------------------------- 
static inline uint8_t RAMFUNC _set(uint8_t num, uint8_t dst)
{
	return (dst|(1<<num)) ;
}


// -------------------------------------------------------------------------------- 
//   RES     num, dst                                                               
// -------------------------------------------------------------------------------- 
static inline uint8_t RAMFUNC _res(uint8_t num, uint8_t dst)
{
	return (uint8_t )(dst&~(1<<num)) ;
}


// -------------------------------------------------------------------------------- 
//   INC     dst                                                                    
// -------------------------------------------------------------------------------- 
static uint8_t RAMFUNC _inc(Z80 *p, uint8_t dst)
{
	uint32_t	res ;


	res         = dst + 1 ;
	p->regs.b.f = (uint8_t )((p->flags[(uint8_t )res]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X)) | ((dst^res)&FLAG_H) | ((((res^dst)&res)>>(7-FLAG_V_))&FLAG_V) | (p->regs.b.f&FLAG_C)) ;

	return (uint8_t )res ;
}


// -------------------------------------------------------------------------------- 
//   DEC     dst                                                                    
// -------------------------------------------------------------------------------- 
static uint8_t RAMFUNC _dec(Z80 *p, uint8_t dst)
{
	uint32_t	res ;


	res         = dst - 1 ;
	p->regs.b.f = (uint8_t )((p->flags[(uint8_t )res]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_N)) | ((dst^res)&FLAG_H) | ((((res^dst)&dst)>>(7-FLAG_V_))&FLAG_V) | (p->regs.b.f&FLAG_C)) ;

	return (uint8_t )res ;
}


// -------------------------------------------------------------------------------- 
//   IN      dst, src                                                               
// -------------------------------------------------------------------------------- 
static inline uint8_t RAMFUNC _in(Z80 *p, uint16_t src)
{
	uint8_t		dst ;


	dst         = readAddressBus(p->p_io, src) ;
	p->regs.b.f = (p->flags[dst]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_P)) | (p->regs.b.f&FLAG_C) ;

	return dst ;
}

static inline void RAMFUNC _in_a_n( Z80 *p )
{
	p->regs.b.a = readAddressBus(p->p_io, ((uint16_t )p->regs.b.a<<8)|readBytePC( p )) ;
}


// -------------------------------------------------------------------------------- 
//   INI / IND                                                                      
// -------------------------------------------------------------------------------- 
static void RAMFUNC _inn(Z80 *p, int16_t add)
{
	uint8_t		data ;


	data = readAddressBus(p->p_io, p->regs.w.bc) ;

	writeByte(p, p->regs.w.hl, data) ;
	p->regs.w.hl = p->regs.w.hl + add ;
	p->regs.b.b-- ;

	p->regs.b.f = ((p->regs.b.b==0)<<FLAG_Z_) | ((data&FLAG_S)>>(FLAG_S_-FLAG_N_)) | (p->regs.b.f&FLAG_C) ;
}


// -------------------------------------------------------------------------------- 
//   INIR / INDR                                                                    
// -------------------------------------------------------------------------------- 
static inline uint32_t RAMFUNC _innr(Z80 *p, int16_t add)
{
	_inn(p, add) ;

	if( !p->regs.b.b ) {
		return _CLK(16) ;
	}

	p->pc -= 2 ;

	return _CLK(21) ;
}


// -------------------------------------------------------------------------------- 
//   OUT     dst, src                                                               
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC _out(Z80 *p, uint16_t dst, uint8_t src)
{
	writeAddressBus(p->p_io, dst, src) ;
}

static inline void RAMFUNC _out_n_a( Z80 *p )
{
	writeAddressBus(p->p_io, ((uint16_t )p->regs.b.a<<8)|readBytePC( p ), p->regs.b.a) ;
}


// -------------------------------------------------------------------------------- 
//   OUTI / OUTD                                                                    
// -------------------------------------------------------------------------------- 
static void RAMFUNC _outn(Z80 *p, int16_t add)
{
	p->regs.b.b-- ;
	p->p_io->write(p->p_io, p->regs.w.bc, readByte(p, p->regs.w.hl)) ;
	p->regs.w.hl = p->regs.w.hl + add ;
	p->regs.b.f  = ((uint8_t )(p->regs.b.b==0)<<FLAG_Z_) | FLAG_N | (p->regs.b.f&FLAG_C) ;
}


// -------------------------------------------------------------------------------- 
//   OTIR / OTDR                                                                    
// -------------------------------------------------------------------------------- 
static inline uint32_t RAMFUNC _otnr(Z80 *p, int16_t add)
{
	_outn(p, add) ;

	if( !p->regs.b.b ) {
		return _CLK(16) ;
	}

	p->pc -= 2 ;

	return _CLK(21) ;
}


// -------------------------------------------------------------------------------- 
//   DAA                                                                            
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC _daa( Z80 *p )
{
	uint8_t		a, flag, diff ;


	flag = p->regs.b.f & (FLAG_N|FLAG_C) ;
	diff = 0x00 ;

	if( (p->regs.b.f&FLAG_H) || ((p->regs.b.a&0x0f)>0x09) ) {
		diff = 0x06 ;
	}

	if( (p->regs.b.f&FLAG_C) || (p->regs.b.a>0x99) ) {
		flag |= FLAG_C ;
		diff |= 0x60 ;
	}

	if( p->regs.b.f & FLAG_N ) {
		diff = -diff ;
	}

	a           = p->regs.b.a ;
	p->regs.b.a = a + diff ;
	p->regs.b.f = flag | (p->flags[p->regs.b.a]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_P)) | ((p->regs.b.a^a)&FLAG_H) ;
}


// -------------------------------------------------------------------------------- 
//   HALT                                                                           
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC _halt( Z80 *p )
{
	if( p->pinIntValue ) {
		p->status |= STATUS_HALT ;
		p->pc-- ;
	}
}


// -------------------------------------------------------------------------------- 
//   DI                                                                             
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC _di( Z80 *p )
{
	p->status &= ~(STATUS_INTREQ|STATUS_INT) ;
	p->iff     = 0 ;
}


// -------------------------------------------------------------------------------- 
//   EI                                                                             
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC _ei( Z80 *p )
{
	p->iff = IFF1 | IFF2 ;

	if( !p->pinIntValue ) {
		p->status |= STATUS_INTREQ ;
	}
}


// -------------------------------------------------------------------------------- 
//   RLCA                                                                           
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC _rlca( Z80 *p )
{
	p->regs.b.a = (p->regs.b.a >> 7) | (p->regs.b.a << 1) ;
	p->regs.b.f = (p->regs.b.f&(FLAG_S|FLAG_Z|FLAG_V)) | (p->regs.b.a&(FLAG_Y|FLAG_X|FLAG_C)) ;
}


// -------------------------------------------------------------------------------- 
//   RLC     dst                                                                    
// -------------------------------------------------------------------------------- 
static uint8_t RAMFUNC _rlc(Z80 *p, uint8_t dst)
{
	dst         = (dst >> 7) | (dst << 1) ;
	p->regs.b.f = (p->flags[dst]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_P)) | (dst&FLAG_C) ;

	return dst ;
}


// -------------------------------------------------------------------------------- 
//   RLA                                                                            
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC _rla( Z80 *p )
{
	uint8_t		cy ;


	cy          = p->regs.b.a >> 7 ;
	p->regs.b.a = (p->regs.b.a << 1) | (p->regs.b.f&FLAG_C) ;
	p->regs.b.f = (p->regs.b.f&(FLAG_S|FLAG_Z|FLAG_V)) | (p->regs.b.a&(FLAG_Y|FLAG_X)) | cy ;
}


// -------------------------------------------------------------------------------- 
//   RL      dst                                                                    
// -------------------------------------------------------------------------------- 
static uint8_t RAMFUNC _rl(Z80 *p, uint8_t dst)
{
	uint8_t		cy ;


	cy          = dst >> 7 ;
	dst         = (dst << 1) | (p->regs.b.f&FLAG_C) ;
	p->regs.b.f = (p->flags[dst]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_P)) | cy ;

	return dst ;
}


// -------------------------------------------------------------------------------- 
//   RRCA                                                                           
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC _rrca( Z80 *p )
{
	p->regs.b.a = (p->regs.b.a << 7) | (p->regs.b.a >> 1) ;
	p->regs.b.f = (p->regs.b.f&(FLAG_S|FLAG_Z|FLAG_V)) | (p->regs.b.a&(FLAG_Y|FLAG_X)) | (p->regs.b.a>>7) ;
}


// -------------------------------------------------------------------------------- 
//   RRC     dst                                                                    
// -------------------------------------------------------------------------------- 
static uint8_t RAMFUNC _rrc(Z80 *p, uint8_t dst)
{
	dst         = (dst << 7) | (dst >> 1) ;
	p->regs.b.f = (p->flags[dst]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_P)) | (dst>>7) ;

	return dst ;
}


// -------------------------------------------------------------------------------- 
//   RRA                                                                            
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC _rra( Z80 *p )
{
	uint8_t		cy ;


	cy          = p->regs.b.a & FLAG_C ;
	p->regs.b.a = (p->regs.b.a >> 1) | (p->regs.b.f << 7) ;
	p->regs.b.f = (p->regs.b.f & (FLAG_S|FLAG_Z|FLAG_V)) | (p->regs.b.a&(FLAG_Y|FLAG_X)) | cy ;
}


// -------------------------------------------------------------------------------- 
//   RR      dst                                                                    
// -------------------------------------------------------------------------------- 
static uint8_t RAMFUNC _rr(Z80 *p, uint8_t dst)
{
	uint8_t		cy ;


	cy          = dst & FLAG_C ;
	dst         = (dst >> 1) | (p->regs.b.f << 7) ;
	p->regs.b.f = (p->flags[dst]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_P)) | cy ;

	return dst ;
}


// -------------------------------------------------------------------------------- 
//   RLD                                                                            
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC _rld( Z80 *p )
{
	uint8_t		dst ;


	dst = readByte(p, p->regs.w.hl) ;
	writeByte(p, p->regs.w.hl, (p->regs.b.a&0x0f)|(dst<<4)) ;

	p->regs.b.a = (p->regs.b.a & 0xf0) | (dst >> 4) ;
	p->regs.b.f = (p->flags[p->regs.b.a]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_P)) | (p->regs.b.f&FLAG_C) ;

}


// -------------------------------------------------------------------------------- 
//   RRD                                                                            
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC _rrd( Z80 *p )
{
	uint8_t		dst ;


	dst = readByte(p, p->regs.w.hl) ;
	writeByte(p, p->regs.w.hl, (p->regs.b.a<<4)|(dst>>4)) ;

	p->regs.b.a = (p->regs.b.a & 0xf0) | (dst & 0x0f) ;
	p->regs.b.f = (p->flags[p->regs.b.a]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_P)) | (p->regs.b.f&FLAG_C) ;
}


// -------------------------------------------------------------------------------- 
//   SLL     dst                                                                    
// -------------------------------------------------------------------------------- 
static uint8_t RAMFUNC _sll(Z80 *p, uint8_t dst)
{
	uint8_t		cy ;


	cy          = dst >> 7 ;
	dst         = (dst << 1) | 1 ;
	p->regs.b.f = (p->flags[dst]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_P)) | cy ;

	return dst ;
}


// -------------------------------------------------------------------------------- 
//   SLA     dst                                                                    
// -------------------------------------------------------------------------------- 
static uint8_t RAMFUNC _sla(Z80 *p, uint8_t dst)
{
	uint8_t		cy ;


	cy          = dst >> 7 ;
	dst       <<= 1 ;
	p->regs.b.f = (p->flags[dst]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_P)) | cy ;

	return dst ;
}


// -------------------------------------------------------------------------------- 
//   SRL     dst                                                                    
// -------------------------------------------------------------------------------- 
static uint8_t RAMFUNC _srl(Z80 *p, uint8_t dst)
{
	uint8_t		cy ;


	cy          = dst & FLAG_C ;
	dst       >>= 1 ;
	p->regs.b.f = (p->flags[dst]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_P)) | cy ;

	return dst ;
}


// -------------------------------------------------------------------------------- 
//   SRA     dst                                                                    
// -------------------------------------------------------------------------------- 
static uint8_t RAMFUNC _sra(Z80 *p, uint8_t dst)
{
	uint8_t		cy ;


	cy          = dst & FLAG_C ;
	dst         = (dst & 0x80) | (dst >> 1) ;
	p->regs.b.f = (p->flags[dst]&(FLAG_S|FLAG_Z|FLAG_Y|FLAG_X|FLAG_P)) | cy ;

	return dst ;
}


// -------------------------------------------------------------------------------- 
//   JP      condition, dst                                                         
// -------------------------------------------------------------------------------- 
static inline uint32_t RAMFUNC _jp(Z80 *p, uint condition)
{
	if( !condition ) {
		p->pc += 2 ;
	} else {
		p->pc  = readWordPC( p ) ;
	}

	return _CLK(10) ;
}


// -------------------------------------------------------------------------------- 
//   JR      condition, dst                                                         
// -------------------------------------------------------------------------------- 
static inline uint32_t RAMFUNC _jr(Z80 *p, uint condition)
{
	int8_t	offset ;


	if( !condition ) {
		p->pc++ ;
		return _CLK(7) ;
	}

	offset  = readBytePC( p ) ;
	p->pc   = p->pc + offset ;
	p->calc = p->pc ;

	return _CLK(12) ;
}


// -------------------------------------------------------------------------------- 
//   DJNZ    dst                                                                    
// -------------------------------------------------------------------------------- 
static inline uint32_t RAMFUNC _djnz( Z80 *p )
{
	int8_t	offset ;


	if( !--p->regs.b.b ) {
		p->pc++ ;
		return _CLK(8) ;
	}

	offset  = readBytePC( p ) ;
	p->pc   = p->pc + offset ;
	p->calc = p->pc ;

	return _CLK(13) ;
}


// -------------------------------------------------------------------------------- 
//   CALL    condition, dst                                                         
// -------------------------------------------------------------------------------- 
static uint32_t RAMFUNC _call(Z80 *p, uint condition)
{
	uint16_t	address ;


	if( !condition ) {
		p->pc += 2 ;
		return _CLK(10) ;
	}

	address = readWordPC( p ) ;
	_push(p, p->pc) ;
	p->pc   = address ;

	return _CLK(17) ;
}


// -------------------------------------------------------------------------------- 
//   RET     condition                                                              
// -------------------------------------------------------------------------------- 
static uint32_t RAMFUNC _ret(Z80 *p, uint condition)
{
	if( !condition ) {
		return _CLK(7) ;
	}

	p->pc = _pop( p ) ;

	return _CLK(11) ;
}


// -------------------------------------------------------------------------------- 
//   RETI                                                                           
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC _reti( Z80 *p )
{
	p->pc = _pop( p ) ;

	if( !p->pinIntValue && (p->iff==(IFF1|IFF2)) ) {
		p->status |= STATUS_INTREQ ;
	}
}


// -------------------------------------------------------------------------------- 
//   RETN                                                                           
// -------------------------------------------------------------------------------- 
static uint32_t RAMFUNC _retn( Z80 *p )
{
	p->pc  = _pop( p ) ;
	p->iff = (p->iff & IFF2) | ((p->iff << 1) & IFF1) ;

	if( !p->pinIntValue && (p->iff==(IFF1|IFF2)) ) {
		p->status |= STATUS_INTREQ ;
	}

	return _CLK(14) ;
}


// -------------------------------------------------------------------------------- 
//   RST     num                                                                    
// -------------------------------------------------------------------------------- 
static inline uint32_t RAMFUNC _rst(Z80 *p, uint16_t num)
{
	_push(p, p->pc) ;
	p->pc = num ;

	return _CLK(11) ;
}


// -------------------------------------------------------------------------------- 
//   CPL                                                                            
// -------------------------------------------------------------------------------- 
static inline void RAMFUNC _cpl( Z80 *p )
{
	p->regs.b.a  = ~p->regs.b.a ;
	p->regs.b.f |= (FLAG_H | FLAG_N | (p->regs.b.a&(FLAG_Y|FLAG_X))) ;
}


// -------------------------------------------------------------------------------- 
//   CB 命令                                                                        
// -------------------------------------------------------------------------------- 
static inline uint32_t RAMFUNC instCodeCB( Z80 *p )
{
	uint8_t		opcode ;


	opcode = readBytePC( p ) ;

	switch( opcode ) {
		CB_RS(0x00, _rlc)
		CB_RS(0x08, _rrc)
		CB_RS(0x10, _rl)
		CB_RS(0x18, _rr)
		CB_RS(0x20, _sla)
		CB_RS(0x28, _sra)
		CB_RS(0x30, _sll)
		CB_RS(0x38, _srl)
		CB_BIT(0x40, 0)
		CB_BIT(0x48, 1)
		CB_BIT(0x50, 2)
		CB_BIT(0x58, 3)
		CB_BIT(0x60, 4)
		CB_BIT(0x68, 5)
		CB_BIT(0x70, 6)
		CB_BIT(0x78, 7)
		CB_BRS(0x80, 0, _res)
		CB_BRS(0x88, 1, _res)
		CB_BRS(0x90, 2, _res)
		CB_BRS(0x98, 3, _res)
		CB_BRS(0xa0, 4, _res)
		CB_BRS(0xa8, 5, _res)
		CB_BRS(0xb0, 6, _res)
		CB_BRS(0xb8, 7, _res)
		CB_BRS(0xc0, 0, _set)
		CB_BRS(0xc8, 1, _set)
		CB_BRS(0xd0, 2, _set)
		CB_BRS(0xd8, 3, _set)
		CB_BRS(0xe0, 4, _set)
		CB_BRS(0xe8, 5, _set)
		CB_BRS(0xf0, 6, _set)
		CB_BRS(0xf8, 7, _set)
	}
}


// -------------------------------------------------------------------------------- 
//   DD/FD CB 命令                                                                  
// -------------------------------------------------------------------------------- 
static inline uint32_t RAMFUNC instCodeXDCB(Z80 *p, uint16_t address)
{
	uint8_t		opcode ;


	address = getId(p, address) ;
	opcode  = readBytePC( p ) ;

	switch( opcode ) {
		XDCB_RS(0x00,  address, _rlc)
		XDCB_RS(0x08,  address, _rrc)
		XDCB_RS(0x10,  address, _rl)
		XDCB_RS(0x18,  address, _rr)
		XDCB_RS(0x20,  address, _sla)
		XDCB_RS(0x28,  address, _sra)
		XDCB_RS(0x30,  address, _sll)
		XDCB_RS(0x38,  address, _srl)
		XDCB_BIT(0x40, address, 0)
		XDCB_BIT(0x48, address, 1)
		XDCB_BIT(0x50, address, 2)
		XDCB_BIT(0x58, address, 3)
		XDCB_BIT(0x60, address, 4)
		XDCB_BIT(0x68, address, 5)
		XDCB_BIT(0x70, address, 6)
		XDCB_BIT(0x78, address, 7)
		XDCB_BRS(0x80, address, 0, _res)
		XDCB_BRS(0x88, address, 1, _res)
		XDCB_BRS(0x90, address, 2, _res)
		XDCB_BRS(0x98, address, 3, _res)
		XDCB_BRS(0xa0, address, 4, _res)
		XDCB_BRS(0xa8, address, 5, _res)
		XDCB_BRS(0xb0, address, 6, _res)
		XDCB_BRS(0xb8, address, 7, _res)
		XDCB_BRS(0xc0, address, 0, _set)
		XDCB_BRS(0xc8, address, 1, _set)
		XDCB_BRS(0xd0, address, 2, _set)
		XDCB_BRS(0xd8, address, 3, _set)
		XDCB_BRS(0xe0, address, 4, _set)
		XDCB_BRS(0xe8, address, 5, _set)
		XDCB_BRS(0xf0, address, 6, _set)
		XDCB_BRS(0xf8, address, 7, _set)
	}
}


// -------------------------------------------------------------------------------- 
//   DD/FD 命令                                                                     
// -------------------------------------------------------------------------------- 
static uint32_t RAMFUNC instCodeXD(Z80 *p, uint16_t *p_reg)
{
	uint8_t		opcode ;
	uint16_t	address ;


	opcode = readBytePC( p ) ;

	switch( opcode ) {
		OP_UNDEF( 0x00 )
		case 0x08: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x09: { *p_reg = _addw(p, *p_reg, p->regs.w.bc) ;                                             return _CLK(15) ;                }	// ADD     IX/IY, BC
		case 0x0a: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x0b: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x0c: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x0d: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x0e: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x0f: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		OP_UNDEF( 0x10 )																														//
		case 0x18: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x19: { *p_reg = _addw(p, *p_reg, p->regs.w.de) ;                                             return _CLK(15) ;                }	// ADD     IX/IY, DE
		case 0x1a: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x1b: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x1c: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x1d: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x1e: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x1f: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x20: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x21: { *p_reg = readWordPC( p ) ;                                                            return _CLK(14) ;                }	// LD      IX/IY, #
		case 0x22: { writeWord(p, readWordPC( p ), *p_reg) ;                                               return _CLK(20) ;                }	// LD      (#16), IX/IY
		case 0x23: { (*p_reg)++ ;                                                                          return _CLK(10) ;                }	// INC     IX/IY
		case 0x24: { *p_reg = ((uint16_t )_inc(p, (uint8_t )(*p_reg>>8)) << 8) | (uint8_t )*p_reg ;        return _CLK(8) ;                 }	// INC     IXH/IYH
		case 0x25: { *p_reg = ((uint16_t )_dec(p, (uint8_t )(*p_reg>>8)) << 8) | (uint8_t )*p_reg ;        return _CLK(8) ;                 }	// DEC     IXH/IYH
		case 0x26: { *p_reg = (*p_reg & 0x00ff) | ((uint16_t )readBytePC( p ) << 8) ;                      return _CLK(10) ;                }	// LD      IXH/IYH, #8
		case 0x27: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x28: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x29: { *p_reg = _addw(p, *p_reg, *p_reg) ;                                                   return _CLK(15) ;                }	// ADD     IX/IY, IX/IY
		case 0x2a: { *p_reg = readWord(p, readWordPC( p )) ;                                               return _CLK(20) ;                }	// LD      IX/IY, (#16)
		case 0x2b: { (*p_reg)-- ;                                                                          return _CLK(10) ;                }	// DEC     IX/IY
		case 0x2c: { *p_reg = (*p_reg & 0xff00) | _inc(p, (uint8_t )*p_reg) ;                              return _CLK(8) ;                 }	// INC     IXL/IYL
		case 0x2d: { *p_reg = (*p_reg & 0xff00) | _dec(p, (uint8_t )*p_reg) ;                              return _CLK(8) ;                 }	// DEC     IXL/IYL
		case 0x2e: { *p_reg = (*p_reg & 0xff00) | readBytePC( p ) ;                                        return _CLK(10) ;                }	// LD      IXL/IYL, #8
		case 0x2f: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x30: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x31: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x32: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x33: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x34: { address = getId(p, *p_reg) ;  writeByte(p, address, _inc(p, readByte(p, address))) ;  return _CLK(23) ;                }	// INC     (IX/IY+d)
		case 0x35: { address = getId(p, *p_reg) ;  writeByte(p, address, _dec(p, readByte(p, address))) ;  return _CLK(23) ;                }	// DEC     (IX/IY+d)
		case 0x36: { address = getId(p, *p_reg) ;  writeByte(p, address, readBytePC( p )) ;                return _CLK(19) ;                }	// LD      (IX/IY+d), #8
		case 0x37: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x38: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x39: { *p_reg = _addw(p, *p_reg, p->regs.w.sp) ;                                             return _CLK(15) ;                }	// ADD     IX/IY, SP
		case 0x3a: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x3b: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x3c: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x3d: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x3e: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x3f: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x40: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x41: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x42: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x43: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x44: { p->regs.b.b = (uint8_t )(*p_reg >> 8) ;                                               return _CLK(8) ;                 }	// LD      B, IXH/IYH
		case 0x45: { p->regs.b.b = (uint8_t )(*p_reg >> 0) ;                                               return _CLK(8) ;                 }	// LD      B, IXL/IYL
		case 0x46: { p->regs.b.b = readByte(p, getId(p, *p_reg)) ;                                         return _CLK(19) ;                }	// LD      B, (IX/IY+d)
		case 0x47: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x48: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x49: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x4a: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x4b: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x4c: { p->regs.b.c = (uint8_t )(*p_reg >> 8) ;                                               return _CLK(8) ;                 }	// LD      C, IXH/IYH
		case 0x4d: { p->regs.b.c = (uint8_t )(*p_reg >> 0) ;                                               return _CLK(8) ;                 }	// LD      C, IXL/IYL
		case 0x4e: { p->regs.b.c = readByte(p, getId(p, *p_reg)) ;                                         return _CLK(19) ;                }	// LD      C, (IX/IY+d)
		case 0x4f: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x50: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x51: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x52: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x53: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x54: { p->regs.b.d = (uint8_t )(*p_reg >> 8) ;                                               return _CLK(8) ;                 }	// LD      D, IXH/IYH
		case 0x55: { p->regs.b.d = (uint8_t )(*p_reg >> 0) ;                                               return _CLK(8) ;                 }	// LD      D, IXL/IYL
		case 0x56: { p->regs.b.d = readByte(p, getId(p, *p_reg)) ;                                         return _CLK(19) ;                }	// LD      D, (IX/IY+d)
		case 0x57: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x58: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x59: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x5a: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x5b: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x5c: { p->regs.b.e = (uint8_t )(*p_reg >> 8) ;                                               return _CLK(8) ;                 }	// LD      E, IXH/IYH
		case 0x5d: { p->regs.b.e = (uint8_t )(*p_reg >> 0) ;                                               return _CLK(8) ;                 }	// LD      E, IXL/IYL
		case 0x5e: { p->regs.b.e = readByte(p, getId(p, *p_reg)) ;                                         return _CLK(19) ;                }	// LD      E, (IX/IY+d)
		case 0x5f: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x60: { *p_reg = (*p_reg & 0x00ff) | ((uint16_t )p->regs.b.b << 8) ;                          return _CLK(8) ;                 }	// LD      IXH/IYH, B
		case 0x61: { *p_reg = (*p_reg & 0x00ff) | ((uint16_t )p->regs.b.c << 8) ;                          return _CLK(8) ;                 }	// LD      IXH/IYH, C
		case 0x62: { *p_reg = (*p_reg & 0x00ff) | ((uint16_t )p->regs.b.d << 8) ;                          return _CLK(8) ;                 }	// LD      IXH/IYH, D
		case 0x63: { *p_reg = (*p_reg & 0x00ff) | ((uint16_t )p->regs.b.e << 8) ;                          return _CLK(8) ;                 }	// LD      IXH/IYH, E
		case 0x64: {                                                                                       return _CLK(8) ;                 }	// LD      IXH/IYH, IXH/IYH
		case 0x65: { *p_reg = (*p_reg & 0x00ff) | ((uint16_t )*p_reg      << 8) ;                          return _CLK(8) ;                 }	// LD      IXH/IYH, IXL/IYL
		case 0x66: { p->regs.b.h = readByte(p, getId(p, *p_reg)) ;                                         return _CLK(19) ;                }	// LD      H, (IX/IY+d)
		case 0x67: { *p_reg = (*p_reg & 0x00ff) | ((uint16_t )p->regs.b.a << 8) ;                          return _CLK(8) ;                 }	// LD      IXH/IYH, A
		case 0x68: { *p_reg = (*p_reg & 0xff00) | p->regs.b.b ;                                            return _CLK(8) ;                 }	// LD      IXL/IYL, B
		case 0x69: { *p_reg = (*p_reg & 0xff00) | p->regs.b.c ;                                            return _CLK(8) ;                 }	// LD      IXL/IYL, C
		case 0x6a: { *p_reg = (*p_reg & 0xff00) | p->regs.b.d ;                                            return _CLK(8) ;                 }	// LD      IXL/IYL, D
		case 0x6b: { *p_reg = (*p_reg & 0xff00) | p->regs.b.e ;                                            return _CLK(8) ;                 }	// LD      IXL/IYL, E
		case 0x6c: { *p_reg = (*p_reg & 0xff00) | (*p_reg >> 8) ;                                          return _CLK(8) ;                 }	// LD      IXL/IYL, IXH/IYH
		case 0x6d: {                                                                                       return _CLK(8) ;                 }	// LD      IXL/IYL, IXL/IYL
		case 0x6e: { p->regs.b.l = readByte(p, getId(p, *p_reg)) ;                                         return _CLK(19) ;                }	// LD      L, (IX/IY+d)
		case 0x6f: { *p_reg = (*p_reg & 0xff00) | p->regs.b.a ;                                            return _CLK(8) ;                 }	// LD      IXL/IYL, A
		case 0x70: { writeByte(p, getId(p, *p_reg), p->regs.b.b) ;                                         return _CLK(19) ;                }	// LD      (IX/IY+d), B
		case 0x71: { writeByte(p, getId(p, *p_reg), p->regs.b.c) ;                                         return _CLK(19) ;                }	// LD      (IX/IY+d), C
		case 0x72: { writeByte(p, getId(p, *p_reg), p->regs.b.d) ;                                         return _CLK(19) ;                }	// LD      (IX/IY+d), D
		case 0x73: { writeByte(p, getId(p, *p_reg), p->regs.b.e) ;                                         return _CLK(19) ;                }	// LD      (IX/IY+d), E
		case 0x74: { writeByte(p, getId(p, *p_reg), p->regs.b.h) ;                                         return _CLK(19) ;                }	// LD      (IX/IY+d), H
		case 0x75: { writeByte(p, getId(p, *p_reg), p->regs.b.l) ;                                         return _CLK(19) ;                }	// LD      (IX/IY+d), L
		case 0x76: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x77: { writeByte(p, getId(p, *p_reg), p->regs.b.a) ;                                         return _CLK(19) ;                }	// LD      (IX/IY+d), A
		case 0x78: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x79: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x7a: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x7b: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x7c: { p->regs.b.a = (uint8_t )(*p_reg >> 8) ;                                               return _CLK(8) ;                 }	// LD      A, IXH/IYH
		case 0x7d: { p->regs.b.a = (uint8_t )(*p_reg >> 0) ;                                               return _CLK(8) ;                 }	// LD      A, IXL/IYL
		case 0x7e: { p->regs.b.a = readByte(p, getId(p, *p_reg)) ;                                         return _CLK(19) ;                }	// LD      A, (IX/IY+d)
		case 0x7f: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x80: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x81: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x82: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x83: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x84: { p->regs.b.a = _addb(p, p->regs.b.a, (uint8_t )(*p_reg>>8)) ;                          return _CLK(8) ;                 }	// ADD     A, IXH/IYH
		case 0x85: { p->regs.b.a = _addb(p, p->regs.b.a, (uint8_t )(*p_reg>>0)) ;                          return _CLK(8) ;                 }	// ADD     A, IXL/IYL
		case 0x86: { p->regs.b.a = _addb(p, p->regs.b.a, readByte(p, getId(p, *p_reg))) ;                  return _CLK(19) ;                }	// ADD     A, (IX/IY+d)
		case 0x87: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x88: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x89: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x8a: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x8b: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x8c: { p->regs.b.a = _adcb(p, p->regs.b.a, (uint8_t )(*p_reg>>8)) ;                          return _CLK(8) ;                 }	// ADC     A, IXH/IYH
		case 0x8d: { p->regs.b.a = _adcb(p, p->regs.b.a, (uint8_t )(*p_reg>>0)) ;                          return _CLK(8) ;                 }	// ADC     A, IXL/IYL
		case 0x8e: { p->regs.b.a = _adcb(p, p->regs.b.a, readByte(p, getId(p, *p_reg))) ;                  return _CLK(19) ;                }	// ADC     A, (IX/IY+d)
		case 0x8f: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x90: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x91: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x92: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x93: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x94: { p->regs.b.a = _sub(p, p->regs.b.a, (uint8_t )(*p_reg>>8)) ;                           return _CLK(8) ;                 }	// SUB     A, IXH/IYH
		case 0x95: { p->regs.b.a = _sub(p, p->regs.b.a, (uint8_t )(*p_reg>>0)) ;                           return _CLK(8) ;                 }	// SUB     A, IXL/IYL
		case 0x96: { p->regs.b.a = _sub(p, p->regs.b.a, readByte(p, getId(p, *p_reg))) ;                   return _CLK(19) ;                }	// SUB     A, (IX/IY+d)
		case 0x97: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x98: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x99: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x9a: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x9b: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0x9c: { p->regs.b.a = _sbcb(p, p->regs.b.a, (uint8_t )(*p_reg>>8)) ;                          return _CLK(8) ;                 }	// SBC     A, IXH/IYH
		case 0x9d: { p->regs.b.a = _sbcb(p, p->regs.b.a, (uint8_t )(*p_reg>>0)) ;                          return _CLK(8) ;                 }	// SBC     A, IXL/IYL
		case 0x9e: { p->regs.b.a = _sbcb(p, p->regs.b.a, readByte(p, getId(p, *p_reg))) ;                  return _CLK(19) ;                }	// SBC     A, (IX/IY+d)
		case 0x9f: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xa0: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xa1: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xa2: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xa3: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xa4: { p->regs.b.a = _and(p, p->regs.b.a, (uint8_t )(*p_reg>>8)) ;                           return _CLK(8) ;                 }	// AND     A, IXH/IYH
		case 0xa5: { p->regs.b.a = _and(p, p->regs.b.a, (uint8_t )(*p_reg>>0)) ;                           return _CLK(8) ;                 }	// AND     A, IXL/IYL
		case 0xa6: { p->regs.b.a = _and(p, p->regs.b.a, readByte(p, getId(p, *p_reg))) ;                   return _CLK(19) ;                }	// AND     A, (IX/IY+d)
		case 0xa7: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xa8: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xa9: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xaa: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xab: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xac: { p->regs.b.a = _xor(p, p->regs.b.a, (uint8_t )(*p_reg>>8)) ;                           return _CLK(8) ;                 }	// XOR     A, IXH/IYH
		case 0xad: { p->regs.b.a = _xor(p, p->regs.b.a, (uint8_t )(*p_reg>>0)) ;                           return _CLK(8) ;                 }	// XOR     A, IXL/IYL
		case 0xae: { p->regs.b.a = _xor(p, p->regs.b.a, readByte(p, getId(p, *p_reg))) ;                   return _CLK(19) ;                }	// XOR     (IX/IY+d)
		case 0xaf: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xb0: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xb1: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xb2: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xb3: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xb4: { p->regs.b.a = _or(p, p->regs.b.a, (uint8_t )(*p_reg>>8)) ;                            return _CLK(8) ;                 }	// OR      IXH/IYH
		case 0xb5: { p->regs.b.a = _or(p, p->regs.b.a, (uint8_t )(*p_reg>>0)) ;                            return _CLK(8) ;                 }	// OR      IXL/IYL
		case 0xb6: { p->regs.b.a = _or(p, p->regs.b.a, readByte(p, getId(p, *p_reg))) ;                    return _CLK(19) ;                }	// OR      (IX/IY+d)
		case 0xb7: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xb8: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xb9: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xba: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xbb: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xbc: { _cp(p, p->regs.b.a, (uint8_t )(*p_reg>>8)) ;                                          return _CLK(8) ;                 }	// CP      IXH/IYH
		case 0xbd: { _cp(p, p->regs.b.a, (uint8_t )(*p_reg>>0)) ;                                          return _CLK(8) ;                 }	// CP      IXL/IYL
		case 0xbe: { _cp(p, p->regs.b.a, readByte(p, getId(p, *p_reg))) ;                                  return _CLK(19) ;                }	// CP      (IX/IY+d)
		case 0xbf: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xc0: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xc1: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xc2: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xc3: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xc4: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xc5: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xc6: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xc7: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xc8: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xc9: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xca: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xcb: {                                                                                       return instCodeXDCB(p, *p_reg) ; }	// DD/FD CB
		case 0xcc: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xcd: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xce: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xcf: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xd0: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xd1: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xd2: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xd3: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xd4: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xd5: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xd6: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xd7: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xd8: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xd9: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xda: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xdb: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xdc: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xdd: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xde: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xdf: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xe0: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xe1: { *p_reg = _pop( p ) ;                                                                  return _CLK(14) ;                }	// POP     IX/IY
		case 0xe2: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xe3: { writeWord(p, p->regs.w.sp, _ex(readWord(p, p->regs.w.sp), p_reg)) ;                   return _CLK(23) ;                }	// EX      (SP), IX/IY
		case 0xe4: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xe5: { _push(p, *p_reg) ;                                                                    return _CLK(15) ;                }	// PUSH    IX/IY
		case 0xe6: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xe7: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xe8: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xe9: { p->pc = *p_reg ;                                                                      return _CLK(8) ;                 }	// JP      (IX/IY)
		case 0xea: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xeb: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xec: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xed: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xee: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xef: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xf0: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xf1: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xf2: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xf3: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xf4: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xf5: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xf6: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xf7: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xf8: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xf9: { p->regs.w.sp = *p_reg ;                                                               return _CLK(10) ;                }	// LD      SP, IX/IY
		case 0xfa: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xfb: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xfc: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xfd: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xfe: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
		case 0xff: { undefInstruction( p ) ;                                                               return _CLK(4) ;                 }	//
	}
}


// -------------------------------------------------------------------------------- 
//   ED 命令                                                                        
// -------------------------------------------------------------------------------- 
static inline uint32_t instCodeED( Z80 *p )
{
	uint8_t		opcode ;


	opcode = readBytePC( p ) ;

	switch( opcode ) {
		OP_UNDEF( 0x00 )																										//
		OP_UNDEF( 0x08 )																										//
		OP_UNDEF( 0x10 )																										//
		OP_UNDEF( 0x18 )																										//
		OP_UNDEF( 0x20 )																										//
		OP_UNDEF( 0x28 )																										//
		OP_UNDEF( 0x30 )																										//
		OP_UNDEF( 0x38 )																										//
		case 0x40: { p->regs.b.b = _in(p, p->regs.w.bc) ;                                             return _CLK(12) ;     }	// IN      B, (C)
		case 0x41: { _out(p, p->regs.w.bc, p->regs.b.b) ;                                             return _CLK(12) ;     }	// OUT     (C), B
		case 0x42: { p->regs.w.hl = _sbcw(p, p->regs.w.hl, p->regs.w.bc) ;                            return _CLK(15) ;     }	// SBC     HL, BC
		case 0x43: { writeWord(p, readWordPC( p ), p->regs.w.bc) ;                                    return _CLK(20) ;     }	// LD      (#16), BC
		case 0x44: {                                                                                  return _neg( p ) ;    }	// NEG
		case 0x45: {                                                                                  return _retn( p ) ;   }	// RETN
		case 0x46: { p->im = 0 ;                                                                      return _CLK(8) ;      }	// IM      0
		case 0x47: { p->i = p->regs.b.a ;                                                             return _CLK(9) ;      }	// LD      I, A
		case 0x48: { p->regs.b.c = _in(p, p->regs.w.bc) ;                                             return _CLK(12) ;     }	// IN      C, (C)
		case 0x49: { _out(p, p->regs.w.bc, p->regs.b.c) ;                                             return _CLK(12) ;     }	// OUT     (C), C
		case 0x4a: { p->calc = p->regs.w.hl ;  p->regs.w.hl = _adcw(p, p->regs.w.hl, p->regs.w.bc) ;  return _CLK(15) ;     }	// ADC     HL, BC
		case 0x4b: { p->regs.w.bc = readWord(p, readWordPC( p )) ;                                    return _CLK(20) ;     }	// LD      BC, (#16)
		case 0x4c: {                                                                                  return _neg( p ) ;    }	// NEG
		case 0x4d: { _reti( p ) ;                                                                     return _CLK(14) ;     }	// RETI
		case 0x4e: { p->im = 0 ;                                                                      return _CLK(8) ;      }	// IM      0
		case 0x4f: { p->r = p->rt = p->regs.b.a ;                                                     return _CLK(9) ;      }	// LD      R, A
		case 0x50: { p->regs.b.d = _in(p, p->regs.w.bc) ;                                             return _CLK(12) ;     }	// IN      D, (C)
		case 0x51: { _out(p, p->regs.w.bc, p->regs.b.d) ;                                             return _CLK(12) ;     }	// OUT     (C), D
		case 0x52: { p->regs.w.hl = _sbcw(p, p->regs.w.hl, p->regs.w.de) ;                            return _CLK(15) ;     }	// SBC     HL, DE
		case 0x53: { writeWord(p, readWordPC( p ), p->regs.w.de) ;                                    return _CLK(20) ;     }	// LD      (#16), DE
		case 0x54: {                                                                                  return _neg( p ) ;    }	// NEG
		case 0x55: {                                                                                  return _retn( p ) ;   }	// RETN
		case 0x56: { p->im = 1 ;                                                                      return _CLK(8) ;      }	// IM      1
		case 0x57: { _lda(p, p->i) ;                                                                  return _CLK(9) ;      }	// LD      A, I
		case 0x58: { p->regs.b.e = _in(p, p->regs.w.bc) ;                                             return _CLK(12) ;     }	// IN      E, (C)
		case 0x59: { _out(p, p->regs.w.bc, p->regs.b.e) ;                                             return _CLK(12) ;     }	// OUT     (C), E
		case 0x5a: { p->calc = p->regs.w.hl ;  p->regs.w.hl = _adcw(p, p->regs.w.hl, p->regs.w.de) ;  return _CLK(15) ;     }	// ADC     HL, DE
		case 0x5b: { p->regs.w.de = readWord(p, readWordPC( p )) ;                                    return _CLK(20) ;     }	// LD      DE, (#16)
		case 0x5c: {                                                                                  return _neg( p ) ;    }	// NEG
		case 0x5d: {                                                                                  return _retn( p ) ;   }	// RETN
		case 0x5e: { p->im = 2 ;                                                                      return _CLK(8) ;      }	// IM      2
		case 0x5f: { p->r = p->rt = (p->rt & 0x80) | ((p->r + 1) & 0x7f) ;  _lda(p, p->r) ;           return _CLK(9) ;      }	// LD      A, R
		case 0x60: { p->regs.b.h = _in(p, p->regs.w.bc) ;                                             return _CLK(12) ;     }	// IN      H, (C)
		case 0x61: { _out(p, p->regs.w.bc, p->regs.b.h) ;                                             return _CLK(12) ;     }	// OUT     (C), H
		case 0x62: { p->regs.w.hl = _sbcw(p, p->regs.w.hl, p->regs.w.hl) ;                            return _CLK(15) ;     }	// SBC     HL, HL
		case 0x63: { writeWord(p, readWordPC( p ), p->regs.w.hl) ;                                    return _CLK(20) ;     }	// LD      (#16), HL
		case 0x64: {                                                                                  return _neg( p ) ;    }	// NEG
		case 0x65: {                                                                                  return _retn( p ) ;   }	// RETN
		case 0x66: { p->im = 0 ;                                                                      return _CLK(8) ;      }	// IM      0
		case 0x67: { _rrd( p ) ;                                                                      return _CLK(18) ;     }	// RRD
		case 0x68: { p->regs.b.l = _in(p, p->regs.w.bc) ;                                             return _CLK(12) ;     }	// IN      L, (C)
		case 0x69: { _out(p, p->regs.w.bc, p->regs.b.l) ;                                             return _CLK(12) ;     }	// OUT     (C), L
		case 0x6a: { p->calc = p->regs.w.hl ;  p->regs.w.hl = _adcw(p, p->regs.w.hl, p->regs.w.hl) ;  return _CLK(15) ;     }	// ADC     HL, HL
		case 0x6b: { p->regs.w.hl = readWord(p, readWordPC( p )) ;                                    return _CLK(20) ;     }	// LD      HL, (#16)
		case 0x6c: {                                                                                  return _neg( p ) ;    }	// NEG
		case 0x6d: {                                                                                  return _retn( p ) ;   }	// RETN
		case 0x6e: { p->im = 0 ;                                                                      return _CLK(8) ;      }	// IM      0
		case 0x6f: { _rld( p ) ;                                                                      return _CLK(18) ;     }	// RLD
		case 0x70: { undefInstruction( p ) ;                                                          return _CLK(11) ;     }	//
		case 0x71: { undefInstruction( p ) ;                                                          return _CLK(12) ;     }	//
		case 0x72: { p->regs.w.hl = _sbcw(p, p->regs.w.hl, p->regs.w.sp) ;                            return _CLK(15) ;     }	// SBC     HL, SP
		case 0x73: { writeWord(p, readWordPC( p ), p->regs.w.sp) ;                                    return _CLK(20) ;     }	// LD      (#16), SP
		case 0x74: {                                                                                  return _neg( p ) ;    }	// NEG
		case 0x75: {                                                                                  return _retn( p ) ;   }	// RETN
		case 0x76: { p->im = 1 ;                                                                      return _CLK(8) ;      }	// IM      1
		case 0x77: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		case 0x78: { p->regs.b.a = _in(p, p->regs.w.bc) ;                                             return _CLK(12) ;     }	// IN      A, (C)
		case 0x79: { _out(p, p->regs.w.bc, p->regs.b.a) ;                                             return _CLK(12) ;     }	// OUT     (C), A
		case 0x7a: { p->calc = p->regs.w.hl ;  p->regs.w.hl = _adcw(p, p->regs.w.hl, p->regs.w.sp) ;  return _CLK(15) ;     }	// ADC     HL, SP
		case 0x7b: { p->regs.w.sp = readWord(p, readWordPC( p )) ;                                    return _CLK(20) ;     }	// LD      SP, (#16)
		case 0x7c: {                                                                                  return _neg( p ) ;    }	// NEG
		case 0x7d: {                                                                                  return _retn( p ) ;   }	// RETN
		case 0x7e: { p->im = 2 ;                                                                      return _CLK(8) ;      }	// IM      2
		case 0x7f: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		OP_UNDEF( 0x80 )																										//
		OP_UNDEF( 0x88 )																										//
		OP_UNDEF( 0x90 )																										//
		OP_UNDEF( 0x98 )																										//
		case 0xa0: { _ldn(p, 1) ;                                                                     return _CLK(16) ;     }	// LDI
		case 0xa1: { _cpn(p, 1) ;                                                                     return _CLK(16) ;     }	// CPI
		case 0xa2: { _inn(p, 1) ;                                                                     return _CLK(16) ;     }	// INI
		case 0xa3: { _outn(p, 1) ;                                                                    return _CLK(16) ;     }	// OUTI
		case 0xa4: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		case 0xa5: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		case 0xa6: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		case 0xa7: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		case 0xa8: { _ldn(p, -1) ;                                                                    return _CLK(16) ;     }	// LDD
		case 0xa9: { _cpn(p, -1) ;                                                                    return _CLK(16) ;     }	// CPD
		case 0xaa: { _inn(p, -1) ;                                                                    return _CLK(16) ;     }	// IND
		case 0xab: { _outn(p, -1) ;                                                                   return _CLK(16) ;     }	// OUTD
		case 0xac: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		case 0xad: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		case 0xae: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		case 0xaf: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		case 0xb0: {                                                                                  return _ldnr(p, 1) ;  }	// LDIR
		case 0xb1: {                                                                                  return _cpnr(p, 1) ;  }	// CPIR
		case 0xb2: {                                                                                  return _innr(p, 1) ;  }	// INIR
		case 0xb3: {                                                                                  return _otnr(p, 1) ;  }	// OTIR
		case 0xb4: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		case 0xb5: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		case 0xb6: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		case 0xb7: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		case 0xb8: {                                                                                  return _ldnr(p, -1) ; }	// LDDR
		case 0xb9: {                                                                                  return _cpnr(p, -1) ; }	// CPDR
		case 0xba: {                                                                                  return _innr(p, -1) ; }	// INDR
		case 0xbb: {                                                                                  return _otnr(p, -1) ; }	// OTDR
		case 0xbc: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		case 0xbd: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		case 0xbe: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		case 0xbf: { undefInstruction( p ) ;                                                          return _CLK(4) ;      }	//
		OP_UNDEF( 0xc0 )																										//
		OP_UNDEF( 0xc8 )																										//
		OP_UNDEF( 0xd0 )																										//
		OP_UNDEF( 0xd8 )																										//
		OP_UNDEF( 0xe0 )																										//
		OP_UNDEF( 0xe8 )																										//
		OP_UNDEF( 0xf0 )																										//
		OP_UNDEF( 0xf8 )																										//
	}
}


// -------------------------------------------------------------------------------- 
//   １バイトオペコード命令                                                         
// -------------------------------------------------------------------------------- 
static inline uint32_t RAMFUNC instCode( Z80 *p )
{
	uint8_t		opcode ;


	opcode = readBytePC( p ) ;

	switch( opcode ) {
		case 0x00: {                                                                                  return _CLK(4) ;                      }	// NOP
		case 0x01: { p->regs.w.bc = readWordPC( p ) ;                                                 return _CLK(10) ;                     }	// LD      BC, #
		case 0x02: { writeByte(p, p->regs.w.bc, p->regs.b.a) ;                                        return _CLK(7) ;                      }	// LD      (BC), A
		case 0x03: { p->regs.w.bc++ ;                                                                 return _CLK(6) ;                      }	// INC     BC
		case 0x04: { p->regs.b.b = _inc(p, p->regs.b.b) ;                                             return _CLK(4) ;                      }	// INC     B
		case 0x05: { p->regs.b.b = _dec(p, p->regs.b.b) ;                                             return _CLK(4) ;                      }	// DEC     B
		case 0x06: { p->regs.b.b = readBytePC( p ) ;                                                  return _CLK(7) ;                      }	// LD      B, #
		case 0x07: { _rlca( p ) ;                                                                     return _CLK(4) ;                      }	// RLCA
		case 0x08: { p->regs.w.af = _ex(p->regs.w.af, &p->regs.w.af_) ;                               return _CLK(4) ;                      }	// EX      AF, AF'
		case 0x09: { p->calc = p->regs.w.hl ;  p->regs.w.hl = _addw(p, p->regs.w.hl, p->regs.w.bc) ;  return _CLK(11) ;                     }	// ADD     HL, BC
		case 0x0a: { p->regs.b.a = readByte(p, p->regs.w.bc) ;                                        return _CLK(7) ;                      }	// LD      A, (BC)
		case 0x0b: { p->regs.w.bc-- ;                                                                 return _CLK(6) ;                      }	// DEC     BC
		case 0x0c: { p->regs.b.c = _inc(p, p->regs.b.c) ;                                             return _CLK(4) ;                      }	// INC     C
		case 0x0d: { p->regs.b.c = _dec(p, p->regs.b.c) ;                                             return _CLK(4) ;                      }	// DEC     C
		case 0x0e: { p->regs.b.c = readBytePC( p ) ;                                                  return _CLK(7) ;                      }	// LD      C, #
		case 0x0f: { _rrca( p ) ;                                                                     return _CLK(4) ;                      }	// RRCA
		case 0x10: {                                                                                  return _djnz( p ) ;                   }	// DJNZ    PC+d8
		case 0x11: { p->regs.w.de = readWordPC( p ) ;                                                 return _CLK(10) ;                     }	// LD      DE, #
		case 0x12: { writeByte(p, p->regs.w.de, p->regs.b.a) ;                                        return _CLK(7) ;                      }	// LD      (DE), A
		case 0x13: { p->regs.w.de++ ;                                                                 return _CLK(6) ;                      }	// INC     DE
		case 0x14: { p->regs.b.d = _inc(p, p->regs.b.d) ;                                             return _CLK(4) ;                      }	// INC     D
		case 0x15: { p->regs.b.d = _dec(p, p->regs.b.d) ;                                             return _CLK(4) ;                      }	// DEC     D
		case 0x16: { p->regs.b.d = readBytePC( p ) ;                                                  return _CLK(7) ;                      }	// LD      D, #
		case 0x17: { _rla( p ) ;                                                                      return _CLK(4) ;                      }	// RLA
		case 0x18: {                                                                                  return _jr(p, CC_T) ;                 }	// JR      PC+d8
		case 0x19: { p->calc = p->regs.w.hl ;  p->regs.w.hl = _addw(p, p->regs.w.hl, p->regs.w.de) ;  return _CLK(11) ;                     }	// ADD     HL, DE
		case 0x1a: { p->regs.b.a = readByte(p, p->regs.w.de) ;                                        return _CLK(7) ;                      }	// LD      A, (DE)
		case 0x1b: { p->regs.w.de-- ;                                                                 return _CLK(6) ;                      }	// DEC     DE
		case 0x1c: { p->regs.b.e = _inc(p, p->regs.b.e) ;                                             return _CLK(4) ;                      }	// INC     E
		case 0x1d: { p->regs.b.e = _dec(p, p->regs.b.e) ;                                             return _CLK(4) ;                      }	// DEC     E
		case 0x1e: { p->regs.b.e = readBytePC( p ) ;                                                  return _CLK(7) ;                      }	// LD      E, #
		case 0x1f: { _rra( p ) ;                                                                      return _CLK(4) ;                      }	// RRA
		case 0x20: {                                                                                  return _jr(p, CC_NZ) ;                }	// JR      NZ, PC+d8
		case 0x21: { p->regs.w.hl = readWordPC( p ) ;                                                 return _CLK(10) ;                     }	// LD      HL, #
		case 0x22: { writeWord(p, readWordPC( p ), p->regs.w.hl) ;                                    return _CLK(16) ;                     }	// LD      (#16), HL
		case 0x23: { p->regs.w.hl++ ;                                                                 return _CLK(6) ;                      }	// INC     HL
		case 0x24: { p->regs.b.h = _inc(p, p->regs.b.h) ;                                             return _CLK(4) ;                      }	// INC     H
		case 0x25: { p->regs.b.h = _dec(p, p->regs.b.h) ;                                             return _CLK(4) ;                      }	// DEC     H
		case 0x26: { p->regs.b.h = readBytePC( p ) ;                                                  return _CLK(7) ;                      }	// LD      H, #
		case 0x27: { _daa( p ) ;                                                                      return _CLK(4) ;                      }	// DAA
		case 0x28: {                                                                                  return _jr(p, CC_Z) ;                 }	// JR      Z, PC+d8
		case 0x29: { p->calc = p->regs.w.hl ;  p->regs.w.hl = _addw(p, p->regs.w.hl, p->regs.w.hl) ;  return _CLK(11) ;                     }	// ADD     HL, HL
		case 0x2a: { p->regs.w.hl = readWord(p, readWordPC( p )) ;                                    return _CLK(16) ;                     }	// LD      HL, (#16)
		case 0x2b: { p->regs.w.hl-- ;                                                                 return _CLK(6) ;                      }	// DEC     HL
		case 0x2c: { p->regs.b.l = _inc(p, p->regs.b.l) ;                                             return _CLK(4) ;                      }	// INC     L
		case 0x2d: { p->regs.b.l = _dec(p, p->regs.b.l) ;                                             return _CLK(4) ;                      }	// DEC     L
		case 0x2e: { p->regs.b.l = readBytePC( p ) ;                                                  return _CLK(7) ;                      }	// LD      L, #
		case 0x2f: { _cpl( p ) ;                                                                      return _CLK(4) ;                      }	// CPL
		case 0x30: {                                                                                  return _jr(p, CC_NC) ;                }	// JR      NC, PC+d8
		case 0x31: { p->regs.w.sp = readWordPC( p ) ;                                                 return _CLK(10) ;                     }	// LD      SP, #
		case 0x32: { writeByte(p, readWordPC( p ), p->regs.b.a) ;                                     return _CLK(13) ;                     }	// LD      (#16), A
		case 0x33: { p->regs.w.sp++ ;                                                                 return _CLK(6) ;                      }	// INC     SP
		case 0x34: { writeByte(p, p->regs.w.hl, _inc(p, readByte(p, p->regs.w.hl))) ;                 return _CLK(11) ;                     }	// INC     (HL)
		case 0x35: { writeByte(p, p->regs.w.hl, _dec(p, readByte(p, p->regs.w.hl))) ;                 return _CLK(11) ;                     }	// DEC     (HL)
		case 0x36: { writeByte(p, p->regs.w.hl, readBytePC( p )) ;                                    return _CLK(10) ;                     }	// LD      (HL), #
		case 0x37: {                                                                                  return _scf( p ) ;                    }	// SCF
		case 0x38: {                                                                                  return _jr(p, CC_C) ;                 }	// JR      C, PC+d8
		case 0x39: { p->calc = p->regs.w.hl ;  p->regs.w.hl = _addw(p, p->regs.w.hl, p->regs.w.sp) ;  return _CLK(11) ;                     }	// ADD     HL, SP
		case 0x3a: { p->regs.b.a = readByte(p, readWordPC( p )) ;                                     return _CLK(13) ;                     }	// LD      A, (#16)
		case 0x3b: { p->regs.w.sp-- ;                                                                 return _CLK(6) ;                      }	// DEC     SP
		case 0x3c: { p->regs.b.a = _inc(p, p->regs.b.a) ;                                             return _CLK(4) ;                      }	// INC     A
		case 0x3d: { p->regs.b.a = _dec(p, p->regs.b.a) ;                                             return _CLK(4) ;                      }	// DEC     A
		case 0x3e: { p->regs.b.a = readBytePC( p ) ;                                                  return _CLK(7) ;                      }	// LD      A, #
		case 0x3f: {                                                                                  return _ccf( p ) ;                    }	// CCF
		case 0x40: { p->regs.b.b = p->regs.b.b ;                                                      return _CLK(4) ;                      }	// LD      B, B
		case 0x41: { p->regs.b.b = p->regs.b.c ;                                                      return _CLK(4) ;                      }	// LD      B, C
		case 0x42: { p->regs.b.b = p->regs.b.d ;                                                      return _CLK(4) ;                      }	// LD      B, D
		case 0x43: { p->regs.b.b = p->regs.b.e ;                                                      return _CLK(4) ;                      }	// LD      B, E
		case 0x44: { p->regs.b.b = p->regs.b.h ;                                                      return _CLK(4) ;                      }	// LD      B, H
		case 0x45: { p->regs.b.b = p->regs.b.l ;                                                      return _CLK(4) ;                      }	// LD      B, L
		case 0x46: { p->regs.b.b = readByte(p, p->regs.w.hl) ;                                        return _CLK(7) ;                      }	// LD      B, (HL)
		case 0x47: { p->regs.b.b = p->regs.b.a ;                                                      return _CLK(4) ;                      }	// LD      B, A
		case 0x48: { p->regs.b.c = p->regs.b.b ;                                                      return _CLK(4) ;                      }	// LD      C, B
		case 0x49: { p->regs.b.c = p->regs.b.c ;                                                      return _CLK(4) ;                      }	// LD      C, C
		case 0x4a: { p->regs.b.c = p->regs.b.d ;                                                      return _CLK(4) ;                      }	// LD      C, D
		case 0x4b: { p->regs.b.c = p->regs.b.e ;                                                      return _CLK(4) ;                      }	// LD      C, E
		case 0x4c: { p->regs.b.c = p->regs.b.h ;                                                      return _CLK(4) ;                      }	// LD      C, H
		case 0x4d: { p->regs.b.c = p->regs.b.l ;                                                      return _CLK(4) ;                      }	// LD      C, L
		case 0x4e: { p->regs.b.c = readByte(p, p->regs.w.hl) ;                                        return _CLK(7) ;                      }	// LD      C, (HL)
		case 0x4f: { p->regs.b.c = p->regs.b.a ;                                                      return _CLK(4) ;                      }	// LD      C, A
		case 0x50: { p->regs.b.d = p->regs.b.b ;                                                      return _CLK(4) ;                      }	// LD      D, B
		case 0x51: { p->regs.b.d = p->regs.b.c ;                                                      return _CLK(4) ;                      }	// LD      D, C
		case 0x52: { p->regs.b.d = p->regs.b.d ;                                                      return _CLK(4) ;                      }	// LD      D, D
		case 0x53: { p->regs.b.d = p->regs.b.e ;                                                      return _CLK(4) ;                      }	// LD      D, E
		case 0x54: { p->regs.b.d = p->regs.b.h ;                                                      return _CLK(4) ;                      }	// LD      D, H
		case 0x55: { p->regs.b.d = p->regs.b.l ;                                                      return _CLK(4) ;                      }	// LD      D, L
		case 0x56: { p->regs.b.d = readByte(p, p->regs.w.hl) ;                                        return _CLK(7) ;                      }	// LD      D, (HL)
		case 0x57: { p->regs.b.d = p->regs.b.a ;                                                      return _CLK(4) ;                      }	// LD      D, A
		case 0x58: { p->regs.b.e = p->regs.b.b ;                                                      return _CLK(4) ;                      }	// LD      E, B
		case 0x59: { p->regs.b.e = p->regs.b.c ;                                                      return _CLK(4) ;                      }	// LD      E, C
		case 0x5a: { p->regs.b.e = p->regs.b.d ;                                                      return _CLK(4) ;                      }	// LD      E, D
		case 0x5b: { p->regs.b.e = p->regs.b.e ;                                                      return _CLK(4) ;                      }	// LD      E, E
		case 0x5c: { p->regs.b.e = p->regs.b.h ;                                                      return _CLK(4) ;                      }	// LD      E, H
		case 0x5d: { p->regs.b.e = p->regs.b.l ;                                                      return _CLK(4) ;                      }	// LD      E, L
		case 0x5e: { p->regs.b.e = readByte(p, p->regs.w.hl) ;                                        return _CLK(7) ;                      }	// LD      E, (HL)
		case 0x5f: { p->regs.b.e = p->regs.b.a ;                                                      return _CLK(4) ;                      }	// LD      E, A
		case 0x60: { p->regs.b.h = p->regs.b.b ;                                                      return _CLK(4) ;                      }	// LD      H, B
		case 0x61: { p->regs.b.h = p->regs.b.c ;                                                      return _CLK(4) ;                      }	// LD      H, C
		case 0x62: { p->regs.b.h = p->regs.b.d ;                                                      return _CLK(4) ;                      }	// LD      H, D
		case 0x63: { p->regs.b.h = p->regs.b.e ;                                                      return _CLK(4) ;                      }	// LD      H, E
		case 0x64: { p->regs.b.h = p->regs.b.h ;                                                      return _CLK(4) ;                      }	// LD      H, H
		case 0x65: { p->regs.b.h = p->regs.b.l ;                                                      return _CLK(4) ;                      }	// LD      H, L
		case 0x66: { p->regs.b.h = readByte(p, p->regs.w.hl) ;                                        return _CLK(7) ;                      }	// LD      H, (HL)
		case 0x67: { p->regs.b.h = p->regs.b.a ;                                                      return _CLK(4) ;                      }	// LD      H, A
		case 0x68: { p->regs.b.l = p->regs.b.b ;                                                      return _CLK(4) ;                      }	// LD      L, B
		case 0x69: { p->regs.b.l = p->regs.b.c ;                                                      return _CLK(4) ;                      }	// LD      L, C
		case 0x6a: { p->regs.b.l = p->regs.b.d ;                                                      return _CLK(4) ;                      }	// LD      L, D
		case 0x6b: { p->regs.b.l = p->regs.b.e ;                                                      return _CLK(4) ;                      }	// LD      L, E
		case 0x6c: { p->regs.b.l = p->regs.b.h ;                                                      return _CLK(4) ;                      }	// LD      L, H
		case 0x6d: { p->regs.b.l = p->regs.b.l ;                                                      return _CLK(4) ;                      }	// LD      L, L
		case 0x6e: { p->regs.b.l = readByte(p, p->regs.w.hl) ;                                        return _CLK(7) ;                      }	// LD      L, (HL)
		case 0x6f: { p->regs.b.l = p->regs.b.a ;                                                      return _CLK(4) ;                      }	// LD      L, A
		case 0x70: { writeByte(p, p->regs.w.hl, p->regs.b.b) ;                                        return _CLK(7) ;                      }	// LD      (HL), B
		case 0x71: { writeByte(p, p->regs.w.hl, p->regs.b.c) ;                                        return _CLK(7) ;                      }	// LD      (HL), C
		case 0x72: { writeByte(p, p->regs.w.hl, p->regs.b.d) ;                                        return _CLK(7) ;                      }	// LD      (HL), D
		case 0x73: { writeByte(p, p->regs.w.hl, p->regs.b.e) ;                                        return _CLK(7) ;                      }	// LD      (HL), E
		case 0x74: { writeByte(p, p->regs.w.hl, p->regs.b.h) ;                                        return _CLK(7) ;                      }	// LD      (HL), H
		case 0x75: { writeByte(p, p->regs.w.hl, p->regs.b.l) ;                                        return _CLK(7) ;                      }	// LD      (HL), L
		case 0x76: { _halt( p ) ;                                                                     return _CLK(4) ;                      }	// HALT
		case 0x77: { writeByte(p, p->regs.w.hl, p->regs.b.a) ;                                        return _CLK(7) ;                      }	// LD      (HL), A
		case 0x78: { p->regs.b.a = p->regs.b.b ;                                                      return _CLK(4) ;                      }	// LD      A, B
		case 0x79: { p->regs.b.a = p->regs.b.c ;                                                      return _CLK(4) ;                      }	// LD      A, C
		case 0x7a: { p->regs.b.a = p->regs.b.d ;                                                      return _CLK(4) ;                      }	// LD      A, D
		case 0x7b: { p->regs.b.a = p->regs.b.e ;                                                      return _CLK(4) ;                      }	// LD      A, E
		case 0x7c: { p->regs.b.a = p->regs.b.h ;                                                      return _CLK(4) ;                      }	// LD      A, H
		case 0x7d: { p->regs.b.a = p->regs.b.l ;                                                      return _CLK(4) ;                      }	// LD      A, L
		case 0x7e: { p->regs.b.a = readByte(p, p->regs.w.hl) ;                                        return _CLK(7) ;                      }	// LD      A, (HL)
		case 0x7f: { p->regs.b.a = p->regs.b.a ;                                                      return _CLK(4) ;                      }	// LD      A, A
		case 0x80: { p->regs.b.a = _addb(p, p->regs.b.a, p->regs.b.b) ;                               return _CLK(4) ;                      }	// ADD     A, B
		case 0x81: { p->regs.b.a = _addb(p, p->regs.b.a, p->regs.b.c) ;                               return _CLK(4) ;                      }	// ADD     A, C
		case 0x82: { p->regs.b.a = _addb(p, p->regs.b.a, p->regs.b.d) ;                               return _CLK(4) ;                      }	// ADD     A, D
		case 0x83: { p->regs.b.a = _addb(p, p->regs.b.a, p->regs.b.e) ;                               return _CLK(4) ;                      }	// ADD     A, E
		case 0x84: { p->regs.b.a = _addb(p, p->regs.b.a, p->regs.b.h) ;                               return _CLK(4) ;                      }	// ADD     A, H
		case 0x85: { p->regs.b.a = _addb(p, p->regs.b.a, p->regs.b.l) ;                               return _CLK(4) ;                      }	// ADD     A, L
		case 0x86: { p->regs.b.a = _addb(p, p->regs.b.a, readByte(p, p->regs.w.hl)) ;                 return _CLK(7) ;                      }	// ADD     A, (HL)
		case 0x87: { p->regs.b.a = _addb(p, p->regs.b.a, p->regs.b.a) ;                               return _CLK(4) ;                      }	// ADD     A, A
		case 0x88: { p->regs.b.a = _adcb(p, p->regs.b.a, p->regs.b.b) ;                               return _CLK(4) ;                      }	// ADC     A, B
		case 0x89: { p->regs.b.a = _adcb(p, p->regs.b.a, p->regs.b.c) ;                               return _CLK(4) ;                      }	// ADC     A, C
		case 0x8a: { p->regs.b.a = _adcb(p, p->regs.b.a, p->regs.b.d) ;                               return _CLK(4) ;                      }	// ADC     A, D
		case 0x8b: { p->regs.b.a = _adcb(p, p->regs.b.a, p->regs.b.e) ;                               return _CLK(4) ;                      }	// ADC     A, E
		case 0x8c: { p->regs.b.a = _adcb(p, p->regs.b.a, p->regs.b.h) ;                               return _CLK(4) ;                      }	// ADC     A, H
		case 0x8d: { p->regs.b.a = _adcb(p, p->regs.b.a, p->regs.b.l) ;                               return _CLK(4) ;                      }	// ADC     A, L
		case 0x8e: { p->regs.b.a = _adcb(p, p->regs.b.a, readByte(p, p->regs.w.hl)) ;                 return _CLK(7) ;                      }	// ADC     A, (HL)
		case 0x8f: { p->regs.b.a = _adcb(p, p->regs.b.a, p->regs.b.a) ;                               return _CLK(4) ;                      }	// ADC     A, A
		case 0x90: { p->regs.b.a = _sub(p, p->regs.b.a, p->regs.b.b) ;                                return _CLK(4) ;                      }	// SUB     B
		case 0x91: { p->regs.b.a = _sub(p, p->regs.b.a, p->regs.b.c) ;                                return _CLK(4) ;                      }	// SUB     C
		case 0x92: { p->regs.b.a = _sub(p, p->regs.b.a, p->regs.b.d) ;                                return _CLK(4) ;                      }	// SUB     D
		case 0x93: { p->regs.b.a = _sub(p, p->regs.b.a, p->regs.b.e) ;                                return _CLK(4) ;                      }	// SUB     E
		case 0x94: { p->regs.b.a = _sub(p, p->regs.b.a, p->regs.b.h) ;                                return _CLK(4) ;                      }	// SUB     H
		case 0x95: { p->regs.b.a = _sub(p, p->regs.b.a, p->regs.b.l) ;                                return _CLK(4) ;                      }	// SUB     L
		case 0x96: { p->regs.b.a = _sub(p, p->regs.b.a, readByte(p, p->regs.w.hl)) ;                  return _CLK(7) ;                      }	// SUB     (HL)
		case 0x97: { p->regs.b.a = _sub(p, p->regs.b.a, p->regs.b.a) ;                                return _CLK(4) ;                      }	// SUB     A
		case 0x98: { p->regs.b.a = _sbcb(p, p->regs.b.a, p->regs.b.b) ;                               return _CLK(4) ;                      }	// SBC     A, B
		case 0x99: { p->regs.b.a = _sbcb(p, p->regs.b.a, p->regs.b.c) ;                               return _CLK(4) ;                      }	// SBC     A, C
		case 0x9a: { p->regs.b.a = _sbcb(p, p->regs.b.a, p->regs.b.d) ;                               return _CLK(4) ;                      }	// SBC     A, D
		case 0x9b: { p->regs.b.a = _sbcb(p, p->regs.b.a, p->regs.b.e) ;                               return _CLK(4) ;                      }	// SBC     A, E
		case 0x9c: { p->regs.b.a = _sbcb(p, p->regs.b.a, p->regs.b.h) ;                               return _CLK(4) ;                      }	// SBC     A, H
		case 0x9d: { p->regs.b.a = _sbcb(p, p->regs.b.a, p->regs.b.l) ;                               return _CLK(4) ;                      }	// SBC     A, L
		case 0x9e: { p->regs.b.a = _sbcb(p, p->regs.b.a, readByte(p, p->regs.w.hl)) ;                 return _CLK(7) ;                      }	// SBC     A, (HL)
		case 0x9f: { p->regs.b.a = _sbcb(p, p->regs.b.a, p->regs.b.a) ;                               return _CLK(4) ;                      }	// SBC     A, A
		case 0xa0: { p->regs.b.a = _and(p, p->regs.b.a, p->regs.b.b) ;                                return _CLK(4) ;                      }	// AND     B
		case 0xa1: { p->regs.b.a = _and(p, p->regs.b.a, p->regs.b.c) ;                                return _CLK(4) ;                      }	// AND     C
		case 0xa2: { p->regs.b.a = _and(p, p->regs.b.a, p->regs.b.d) ;                                return _CLK(4) ;                      }	// AND     D
		case 0xa3: { p->regs.b.a = _and(p, p->regs.b.a, p->regs.b.e) ;                                return _CLK(4) ;                      }	// AND     E
		case 0xa4: { p->regs.b.a = _and(p, p->regs.b.a, p->regs.b.h) ;                                return _CLK(4) ;                      }	// AND     H
		case 0xa5: { p->regs.b.a = _and(p, p->regs.b.a, p->regs.b.l) ;                                return _CLK(4) ;                      }	// AND     L
		case 0xa6: { p->regs.b.a = _and(p, p->regs.b.a, readByte(p, p->regs.w.hl)) ;                  return _CLK(7) ;                      }	// AND     (HL)
		case 0xa7: { p->regs.b.a = _and(p, p->regs.b.a, p->regs.b.a) ;                                return _CLK(4) ;                      }	// AND     A
		case 0xa8: { p->regs.b.a = _xor(p, p->regs.b.a, p->regs.b.b) ;                                return _CLK(4) ;                      }	// XOR     B
		case 0xa9: { p->regs.b.a = _xor(p, p->regs.b.a, p->regs.b.c) ;                                return _CLK(4) ;                      }	// XOR     C
		case 0xaa: { p->regs.b.a = _xor(p, p->regs.b.a, p->regs.b.d) ;                                return _CLK(4) ;                      }	// XOR     D
		case 0xab: { p->regs.b.a = _xor(p, p->regs.b.a, p->regs.b.e) ;                                return _CLK(4) ;                      }	// XOR     E
		case 0xac: { p->regs.b.a = _xor(p, p->regs.b.a, p->regs.b.h) ;                                return _CLK(4) ;                      }	// XOR     H
		case 0xad: { p->regs.b.a = _xor(p, p->regs.b.a, p->regs.b.l) ;                                return _CLK(4) ;                      }	// XOR     L
		case 0xae: { p->regs.b.a = _xor(p, p->regs.b.a, readByte(p, p->regs.w.hl)) ;                  return _CLK(7) ;                      }	// XOR     (HL)
		case 0xaf: { p->regs.b.a = _xor(p, p->regs.b.a, p->regs.b.a) ;                                return _CLK(4) ;                      }	// XOR     A
		case 0xb0: { p->regs.b.a = _or(p, p->regs.b.a, p->regs.b.b) ;                                 return _CLK(4) ;                      }	// OR      B
		case 0xb1: { p->regs.b.a = _or(p, p->regs.b.a, p->regs.b.c) ;                                 return _CLK(4) ;                      }	// OR      C
		case 0xb2: { p->regs.b.a = _or(p, p->regs.b.a, p->regs.b.d) ;                                 return _CLK(4) ;                      }	// OR      D
		case 0xb3: { p->regs.b.a = _or(p, p->regs.b.a, p->regs.b.e) ;                                 return _CLK(4) ;                      }	// OR      E
		case 0xb4: { p->regs.b.a = _or(p, p->regs.b.a, p->regs.b.h) ;                                 return _CLK(4) ;                      }	// OR      H
		case 0xb5: { p->regs.b.a = _or(p, p->regs.b.a, p->regs.b.l) ;                                 return _CLK(4) ;                      }	// OR      L
		case 0xb6: { p->regs.b.a = _or(p, p->regs.b.a, readByte(p, p->regs.w.hl)) ;                   return _CLK(7) ;                      }	// OR      (HL)
		case 0xb7: { p->regs.b.a = _or(p, p->regs.b.a, p->regs.b.a) ;                                 return _CLK(4) ;                      }	// OR      A
		case 0xb8: { _cp(p, p->regs.b.a, p->regs.b.b) ;                                               return _CLK(4) ;                      }	// CP      A, B
		case 0xb9: { _cp(p, p->regs.b.a, p->regs.b.c) ;                                               return _CLK(4) ;                      }	// CP      A, C
		case 0xba: { _cp(p, p->regs.b.a, p->regs.b.d) ;                                               return _CLK(4) ;                      }	// CP      A, D
		case 0xbb: { _cp(p, p->regs.b.a, p->regs.b.e) ;                                               return _CLK(4) ;                      }	// CP      A, E
		case 0xbc: { _cp(p, p->regs.b.a, p->regs.b.h) ;                                               return _CLK(4) ;                      }	// CP      A, H
		case 0xbd: { _cp(p, p->regs.b.a, p->regs.b.l) ;                                               return _CLK(4) ;                      }	// CP      A, L
		case 0xbe: { _cp(p, p->regs.b.a, readByte(p, p->regs.w.hl)) ;                                 return _CLK(7) ;                      }	// CP      A, (HL)
		case 0xbf: { _cp(p, p->regs.b.a, p->regs.b.a) ;                                               return _CLK(4) ;                      }	// CP      A, A
		case 0xc0: {                                                                                  return _ret(p, CC_NZ) ;               }	// RET     NZ
		case 0xc1: { p->regs.w.bc = _pop( p ) ;                                                       return _CLK(10) ;                     }	// POP     BC
		case 0xc2: {                                                                                  return _jp(p, CC_NZ) ;                }	// JP      NZ, #16
		case 0xc3: {                                                                                  return _jp(p, CC_T) ;                 }	// JP      #16
		case 0xc4: {                                                                                  return _call(p, CC_NZ) ;              }	// CALL    NZ, #16
		case 0xc5: { _push(p, p->regs.w.bc) ;                                                         return _CLK(11) ;                     }	// PUSH    BC
		case 0xc6: { p->regs.b.a = _addb(p, p->regs.b.a, readBytePC( p )) ;                           return _CLK(7) ;                      }	// ADD     A, #
		case 0xc7: {                                                                                  return _rst(p, 0x00) ;                }	// RST     00h
		case 0xc8: {                                                                                  return _ret(p, CC_Z) ;                }	// RET     Z
		case 0xc9: { _ret(p, CC_T) ;                                                                  return _CLK(10) ;                     }	// RET
		case 0xca: {                                                                                  return _jp(p, CC_Z) ;                 }	// JP      Z, #16
		case 0xcb: {                                                                                  return instCodeCB( p ) ;              }	// CB 命令
		case 0xcc: {                                                                                  return _call(p, CC_Z) ;               }	// CALL    Z, #16
		case 0xcd: { _call(p, CC_T) ;                                                                 return _CLK(17) ;                     }	// CALL    #16
		case 0xce: { p->regs.b.a = _adcb(p, p->regs.b.a, readBytePC( p )) ;                           return _CLK(7) ;                      }	// ADC     A, #
		case 0xcf: {                                                                                  return _rst(p, 0x08) ;                }	// RST     08h
		case 0xd0: {                                                                                  return _ret(p, CC_NC) ;               }	// RET     NC
		case 0xd1: { p->regs.w.de = _pop( p ) ;                                                       return _CLK(10) ;                     }	// POP     DE
		case 0xd2: {                                                                                  return _jp(p, CC_NC) ;                }	// JP      NC, #16
		case 0xd3: { _out_n_a( p ) ;                                                                  return _CLK(11) ;                     }	// OUT     (#8), A
		case 0xd4: {                                                                                  return _call(p, CC_NC) ;              }	// CALL    NC, #16
		case 0xd5: { _push(p, p->regs.w.de) ;                                                         return _CLK(11) ;                     }	// PUSH    DE
		case 0xd6: { p->regs.b.a = _sub(p, p->regs.b.a, readBytePC( p )) ;                            return _CLK(7) ;                      }	// SUB     #
		case 0xd7: {                                                                                  return _rst(p, 0x10) ;                }	// RST     10h
		case 0xd8: {                                                                                  return _ret(p, CC_C) ;                }	// RET     C
		case 0xd9: {                                                                                  return _exx( p ) ;                    }	// EXX
		case 0xda: {                                                                                  return _jp(p, CC_C) ;                 }	// JP      C, #16
		case 0xdb: { _in_a_n( p ) ;                                                                   return _CLK(11) ;                     }	// IN      A, (#8)
		case 0xdc: {                                                                                  return _call(p, CC_C) ;               }	// CALL    C, #16
		case 0xdd: {                                                                                  return instCodeXD(p, &p->regs.w.ix) ; }	// DD 命令
		case 0xde: { p->regs.b.a = _sbcb(p, p->regs.b.a, readBytePC( p )) ;                           return _CLK(7) ;                      }	// SBC     A, #
		case 0xdf: {                                                                                  return _rst(p, 0x18) ;                }	// RST     18h
		case 0xe0: {                                                                                  return _ret(p, CC_PO) ;               }	// RET     PO
		case 0xe1: { p->regs.w.hl = _pop( p ) ;                                                       return _CLK(10) ;                     }	// POP     HL
		case 0xe2: {                                                                                  return _jp(p, CC_PO) ;                }	// JP      PO, #16
		case 0xe3: { writeWord(p, p->regs.w.sp, _ex(readWord(p, p->regs.w.sp), &p->regs.w.hl)) ;      return _CLK(19) ;                     }	// EX      (SP), HL
		case 0xe4: {                                                                                  return _call(p, CC_PO) ;              }	// CALL    PO, #16
		case 0xe5: { _push(p, p->regs.w.hl) ;                                                         return _CLK(11) ;                     }	// PUSH    HL
		case 0xe6: { p->regs.b.a = _and(p, p->regs.b.a, readBytePC( p )) ;                            return _CLK(7) ;                      }	// AND     #
		case 0xe7: {                                                                                  return _rst(p, 0x20) ;                }	// RST     20h
		case 0xe8: {                                                                                  return _ret(p, CC_PE) ;               }	// RET     PE
		case 0xe9: { p->pc = p->regs.w.hl ;                                                           return _CLK(4) ;                      }	// JP      (HL)
		case 0xea: {                                                                                  return _jp(p, CC_PE) ;                }	// JP      PE, #16
		case 0xeb: { p->regs.w.de = _ex(p->regs.w.de, &p->regs.w.hl) ;                                return _CLK(4) ;                      }	// EX      DE, HL
		case 0xec: {                                                                                  return _call(p, CC_PE) ;              }	// CALL    PE, #16
		case 0xed: {                                                                                  return instCodeED( p ) ;              }	// ED 命令
		case 0xee: { p->regs.b.a = _xor(p, p->regs.b.a, readBytePC( p )) ;                            return _CLK(7) ;                      }	// XOR     #
		case 0xef: {                                                                                  return _rst(p, 0x28) ;                }	// RST     28h
		case 0xf0: {                                                                                  return _ret(p, CC_P) ;                }	// RET     P
		case 0xf1: { p->regs.w.af = _pop( p ) ;                                                       return _CLK(10) ;                     }	// POP     AF
		case 0xf2: {                                                                                  return _jp(p, CC_P) ;                 }	// JP      P, #16
		case 0xf3: { _di( p ) ;                                                                       return _CLK(4) ;                      }	// DI
		case 0xf4: {                                                                                  return _call(p, CC_P) ;               }	// CALL    P, #16
		case 0xf5: { _push(p, p->regs.w.af) ;                                                         return _CLK(11) ;                     }	// PUSH    AF
		case 0xf6: { p->regs.b.a = _or(p, p->regs.b.a, readBytePC( p )) ;                             return _CLK(7) ;                      }	// OR      #
		case 0xf7: {                                                                                  return _rst(p, 0x30) ;                }	// RST     30h
		case 0xf8: {                                                                                  return _ret(p, CC_M) ;                }	// RET     M
		case 0xf9: { p->regs.w.sp = p->regs.w.hl ;                                                    return _CLK(6) ;                      }	// LD      SP, HL
		case 0xfa: {                                                                                  return _jp(p, CC_M) ;                 }	// JP      M, #16
		case 0xfb: { _ei( p ) ;                                                                       return _CLK(4) ;                      }	// EI
		case 0xfc: {                                                                                  return _call(p, CC_M) ;               }	// CALL    M, #16
		case 0xfd: {                                                                                  return instCodeXD(p, &p->regs.w.iy) ; }	// FD 命令
		case 0xfe: { _cp(p, p->regs.b.a, readBytePC( p )) ;                                           return _CLK(7) ;                      }	// CP      #
		case 0xff: {                                                                                  return _rst(p, 0x38) ;                }	// RST     38h
	}
}


// -------------------------------------------------------------------------------- 
//!  NMI ピン書き込み                                                               
//!                                                                                 
//!  @param [in,out] p_this  this ポインタ                                          
//!  @param [in]     value   値                                                     
//!  @return                 なし                                                   
// -------------------------------------------------------------------------------- 
static void pin_write_nmi(PIN *p_this, int16_t value)
{
	Z80	*p=_THISPTR(Z80, pinNmi, p_this) ;


	value = (value > 0) ;

	if( p->pinNmiValue == value ) {
		return ;
	}

	p->pinNmiValue = value ;

	if( !value ) {
		p->status |= STATUS_NMIREQ ;
	}
}


// -------------------------------------------------------------------------------- 
//!  INT ピン書き込み                                                               
//!                                                                                 
//!  @param [in,out] p_this  this ポインタ                                          
//!  @param [in]     value   値                                                     
//!  @return                 なし                                                   
// -------------------------------------------------------------------------------- 
static void pin_write_int(PIN *p_this, int16_t value)
{
	Z80		*p=_THISPTR(Z80, pinInt, p_this) ;


	value = (value > 0) ;

	if( p->pinIntValue == value ) {
		return ;
	}

	p->pinIntValue = value ;

	p->status &= ~STATUS_INTREQ ;
	if( !value && (p->iff==(IFF1|IFF2)) ) {
		p->status |= STATUS_INTREQ ;
	}
}


// -------------------------------------------------------------------------------- 
//!  クロックイベント処理                                                           
//!                                                                                 
//!  @param [in,out] p_this    this ポインタ                                        
//!  @retval         0         クロックイベント破棄                                 
//!  @retval         > 0       クロック数                                           
//!  @retval         < 0       クロック数（符号反転）＋クロックジェネレータ停止     
// -------------------------------------------------------------------------------- 
static int32_t RAMFUNC clockevent_process( CLOCKEVENT *p_this )
{
	Z80			*p=_THISPTR(Z80, clockEvent, p_this) ;
	uint8_t		status ;


//	p->r = (p->r & 0x80) | ((p->r + 1) & 0x7f) ;
	p->r++ ;

	status = p->status ;
	if( status ) {
		if( !(status&STATUS_INTMASK) ) {
			p->status = (p->status & ~(STATUS_NMIREQ|STATUS_INTREQ)) | (status >> STATUS_REQ_) ;

			// NMI
			if( status & STATUS_NMI ) {

				// HALT 解除
				if( p->status & STATUS_HALT ) {
					p->status &= ~STATUS_HALT ;
					p->pc++ ;
				}

				p->status &= ~STATUS_NMI ;

				_push(p, p->pc) ;
				p->pc   = 0x0066 ;
				p->iff &= ~IFF1 ;

				return _CLK(11) ;
			}

			// IRQ
			if( (status&STATUS_INT) && (p->iff==(IFF1|IFF2)) ) {

				// HALT 解除
				if( p->status & STATUS_HALT ) {
					p->status &= ~STATUS_HALT ;
					p->pc++ ;
				}

				p->status &= ~STATUS_INT ;
				p->iff     = 0 ;

				switch( p->im ) {
					case 0:
					{
						return _CLK(13) ;
					}

					case 1:
					{
						_push(p, p->pc) ;
						p->pc = 0x0038 ;

						return _CLK(13) ;
					}

					case 2:
					{
						return _CLK(19) ;
					}
				}
			}
		}

		// HALT 中
		if( status & STATUS_HALT ) {
			return _CLK(4) ;
		}
	}

	return instCode( p ) ;
}


// -------------------------------------------------------------------------------- 
//!  初期化                                                                         
//!                                                                                 
//!  @param [in,out] p      this ポインタ                                           
//!  @param [in]     p_io   I/O アドレスバス                                        
//!  @param [in]     p_mem  メモリーアドレスバス                                    
//!  @return                なし                                                    
// -------------------------------------------------------------------------------- 
void z80_init(Z80*p, ADDRESSBUS *p_io, ADDRESSBUS *p_mem)
{
	uint	index, parity ;


	p->pinNmi.write       = pin_write_nmi ;
	p->pinInt.write       = pin_write_int ;
	p->pinNmiValue        = 1 ;
	p->pinIntValue        = 1 ;
	p->p_io               = p_io ;
	p->p_mem              = p_mem ;
	p->clockEvent.process = clockevent_process ;

	// フラグ作成
	for(index=0; index<256; index++) {

		// パリティ
		parity = ((index >> 0) + (index >> 1) + (index >> 2) + (index >> 3) + (index >> 4) + (index >> 5) + (index >> 6) + (index >> 7)) & 1 ;

		p->flags[index] = (uint8_t )(FLAG_H | FLAG_N | (index & (FLAG_Y|FLAG_X)) | FLAG_C) ;
		if( index & 0x80 ) {
			p->flags[index] |= FLAG_S ;
		}
		if( !index ) {
			p->flags[index] |= FLAG_Z ;
		}	
		if( !parity ) {
			p->flags[index] |= FLAG_P ;
		}
	}
}


// -------------------------------------------------------------------------------- 
//!  RESET                                                                          
//!                                                                                 
//!  @param [in,out] p  this ポインタ                                               
//!  @return            なし                                                        
// -------------------------------------------------------------------------------- 
void z80_reset( Z80 *p )
{
	memset(&p->regs, 0, sizeof( p->regs )) ;
	p->pc        = 0x0000 ;
	p->status    = 0 ;
	p->r         = 0 ;
	p->rt        = 0 ;
	p->i         = 0 ;
	p->im        = 1 ;
	p->iff       = 0 ;
	p->calc      = 0 ;
	p->regs.w.sp = 0xfffc ;

	clockgen_attachEvent(&p->clockEvent, 1) ;
}
